--
-- Mod: FS19_RealLifeNumbers by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- Script: RealNumbersCommodityPrices by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- e-mail: kaj-aage.henneberg@outlook.dk
-- All rights reserved.
-- Version 1.0.0.0
-- 30.11.2018 - 20.03.2019

-- Names for FillTypes

-- WHEAT BARLEY OAT CANOLA SUNFLOWER SOYBEAN MAIZE POTATO SUGARBEET OILSEEDRADISH
-- MANURE SEEDS FORAGE FORAGE_MIXING CHAFF WOODCHIPS SILAGE
-- STRAW GRASS_WINDROW DRYGRASS_WINDROW SUGARCANE FERTILIZER PIGFOOD LIME
-- WOOL TREESAPLINGS EGG
-- MILK FUEL WATER DIESEL DEF
-- LIQUIDFERTILIZER HERBICIDE
-- CHAFF GRASS_WINDROW
-- LIQUIDMANURE DIGESTATE
-- COW_TYPE_BRAHMAN_GREY COW_TYPE_BRAHMAN_LIGHT_BROWN COW_TYPE_BRAHMAN_WHITE
-- COW_TYPE_BRAHMAN_BROWN COW_TYPE_BROWN COW_TYPE_BROWN_WHITE COW_TYPE_BLACK
-- COW_TYPE_BLACK_WHITE
-- SHEEP_TYPE_WHITE SHEEP_TYPE_BROWN SHEEP_TYPE_BLACK_WHITE SHEEP_TYPE_BLACK
-- PIG_TYPE_RED PIG_TYPE_WHITE PIG_TYPE_BLACK_WHITE PIG_TYPE_BLACK
-- CHICKEN_TYPE_BLACK CHICKEN_TYPE_WHITE CHICKEN_TYPE_BROWN CHICKEN_TYPE_ROOSTER

--[[
Commodity prices

Farmland

	Price per hectar (farmland) is obtained from EUROSTAT for different EU countries.
	FS19 operates with two areas (1) farmland area and (2) field area. When a field is bought, you buy a larger piece of land, the farmland.
	The field area is only the area tillaged for crop growth. The purchase price for the ntire farmland is the pricePerHa * farmlandArea. The
	in-game mission page will give information about field area, but not the farmland are. To get information about both use the script
	RealNumbersFieldInfo included in this mod. To use it press ,LeftAlt-rf. Information is printed to the log file.

Fuel

	Fuel and diesel: National averagage values.

Seed

	Seed price is calculated as follow. From the US department of agriculture (USDA) statistical service seed prices were collected as national
	average values (2010 - 2014) for spring wheat, winter wheat, barley, oat, sunflower, soybean, and maize, and scaled to $/100 kg. For each crop, these numbers were
	multiplied by the seed usage for that crop to calculate the $/ha for each crop. Adding up the seed cost for each crop gives the seed cost for
	1 ha of each crop. The seed usage for each crop was converted from mass to volume by dividing the mass amount per ha by 50% of the fresh crop
	mass density (assuming that seeds contain less water than fresh crop). Dividing the total seed cost by the total seed volume gives the
	seed price per liter. Dividing the total seed mass by the total seed volume gives a seed density. Potatoes have been excluded from this calulation
	because seed usage and seed density for potatoes differs significantly from grain type seeds and would bias the result heavily toward potatoes.

	Seed price: 0.3626 $/liter (default ingame: 0.3 pricePerLiter)
	Seed density: 0.3207 kg/Liter (default ingame: 0.35 massPerLiter)

Spray types

	Fertilizer prices are averaged prices over several NPK type fertilizers, as different countries use different NPK mix.

	Liquid fertilizer: data are not published by EUROSTAT or USDA. Default in-game values are used.

	Lime data are only available as US national averages. US values are also used for EU.

	Herbicide data are only available as US national averages. US values are also used for EU.

Feed

	There are no straw prices at USDA. The average value for EU is used.

	Hay prices are purchase pricess for EU and Sell prices for US.

	Pig food prices is an average over a variety of complete feed types

Final comments

	Some price parameters have not been changed yet, as I have not been able to find the tables, in which
	they are stored:

	Annual loan interest is 100 percent by defult. This is much too high. Less than 5-8 % is more realistic.
	Helper salery is very high. Should be 8 - 12 €/hour in some countries.

	Animal buy and sell prices are defined in the script RealNumbersAnimalTradePrices also included in this mod.

]]--


RealNumbersCommodityPrices = {};


function RealNumbersCommodityPrices:loadMap(name)
	local currency = "€";	-- or  "$" or "£"
    local textWidth = 40; 	-- text width in log file
    local iFillType = 0;
	local storeObject = 0;
	local Item = "Empty";
	local idx = 0;
	local price = 0;
	local pricePerHa = 0;
	local massScale = 1000;
	local randomness = 1; 			--1: on, 0: off
	local isDebug = RN.isDebug;  	-- Change to False when not debugging

	local Commodity = RN.commodity;
	local commodityDensity = RN.commodityDensity;
	local commodityPricePer100kg = RN.commodityPricePer100kg;
	local randomPriceFactor = 1;
	local roundBales = RN.roundBales;
	local roundBaleMass = RN.roundBaleMass;
	local roundBaleVolume = RN.roundBaleVolume;
	local roundBalePricePer100kg = RN.roundBalePricePer100kg;
	local squareBales = RN.squareBales;
	local squareBaleMass = RN.squareBaleMass;
	local squareBaleVolume = RN.squareBaleVolume;
	local squareBalePricePer100kg = RN.squareBalePricePer100kg;
	local organicMaterials = RN.organicMaterials;
	local organicPrices = RN.organicPrices;


	local liqfertDilutionFactor = 20;											-- 1 Liter of liquid fertilizer is diluted to 20 liters of solution
	local herbicideDilutionFactor = 200;										-- 1 liter of herbicide is diluted to 200 liters of solution
	local gameMoneyUnit = g_i18n.moneyUnit;
	if gameMoneyUnit == 1 then
        currency = "€";
	elseif gameMoneyUnit == 2 then
        currency = "$";
	elseif gameMoneyUnit == 3 then
        currency = "£";
	end;
	RN.fieldPriceList = {};
	RN.fieldStatus = {};														-- For sale, For rent, Owned, Rented
	RN.sprayCost = {};

	for k = 1, 100 do
		RN.fieldPriceList[k] = RN.pricePerHa;									-- list of purchase prices per hectare
		RN.fieldStatus[k] = " For sale";
	end;
	for k = 1, #RN.rentedFields do
		RN.fieldPriceList[RN.rentedFields[k]] = RN.rentPricePerHa; 				-- overwrite purchase prices with rent prices on rentable field
		RN.fieldStatus[RN.rentedFields[k]] = " For rent";
	end;


	print("FS19_RealLifeNumbers: RealNumbersCommodityPrices active")




	if isDebug then print("\nRealNumbersCommodityPrices Debug Report Start \n") end;


-- Interest rate

	if isDebug then
		myPrint("Annual interest rate", RN.annualLoanInterest,textWidth,"%");
		myPrint("Max loan", RN.loanMax,textWidth,currency);
		end;

-- Farmland

	g_farmlandManager.pricePerHa = RN.pricePerHa;  -- set new value value
	local pricePerHa = g_farmlandManager.pricePerHa;  -- read new value
	if isDebug then
		myPrint("Farmland purchase price Per Ha", pricePerHa,textWidth,currency.."/ha");
		myPrint("Farmland rent price Per Ha", RN.rentPricePerHa,textWidth,currency.."/ha");
		print(" ");
	end;

	-- setting the price of each farmland must wait till map has finished loading, hence placed in update() function below

-- FUEL

    idx = 1;
    Item = Commodity[idx];
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	price = randomPriceFactor * commodityDensity[idx]*commodityPricePer100kg[idx]/100;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('FUEL');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = commodityDensity[idx]/massScale;
    g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].previousHourPrice = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].startPricePerLiter = price;

	if isDebug then
		myPrint(Item.." fillTypeIndex",iFillType,textWidth," ");
		myPrintFloat(Item.." massPerLiter",commodityDensity[idx],textWidth," kg/L");
		myPrintFloat("RandomPriceFactor",randomPriceFactor,textWidth," ");
		myPrint(Item.." price",1000*price,textWidth,currency.."/1000 L");
		print(" ");
	end;


-- WATER

    idx = 2;
    Item = Commodity[idx];
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	price = randomPriceFactor * commodityDensity[idx]*commodityPricePer100kg[idx]/100;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('WATER');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = commodityDensity[idx]/massScale;
    g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].previousHourPrice = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].startPricePerLiter = price;
	if isDebug then
		myPrint(Item.." fillTypeIndex",iFillType,textWidth," ");
		myPrintFloat(Item.." massPerLiter",commodityDensity[idx],textWidth," kg/L");
		myPrintFloat("RandomPriceFactor",randomPriceFactor,textWidth," ");
		myPrint(Item.." price",1000*price,textWidth,currency.."/1000 L");
		print(" ");
	end;


-- SEEDS

    idx = 3;
    Item = Commodity[idx];
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	price = randomPriceFactor * commodityDensity[idx]*commodityPricePer100kg[idx]/100;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('SEEDS');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = commodityDensity[idx]/massScale;
    g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].previousHourPrice = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].startPricePerLiter = price;
	if isDebug then
		myPrint(Item.." fillTypeIndex",iFillType,textWidth," ");
		myPrintFloat(Item.." massPerLiter",commodityDensity[idx],textWidth," kg/L");
		myPrintFloat("RandomPriceFactor",randomPriceFactor,textWidth," ");
		myPrint(Item.." price",1000*price,textWidth,currency.."/1000 L");
		print(" ");
	end;


-- DIESEL

    idx = 4;
    Item = Commodity[idx];
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	price = randomPriceFactor * commodityDensity[idx]*commodityPricePer100kg[idx]/100;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('DIESEL');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = commodityDensity[idx]/massScale;
    g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].previousHourPrice = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].startPricePerLiter = price;
	if isDebug then
		myPrint(Item.." fillTypeIndex",iFillType,textWidth," ");
		myPrintFloat(Item.." massPerLiter",commodityDensity[idx],textWidth," kg/L");
		myPrintFloat("RandomPriceFactor",randomPriceFactor,textWidth," ");
		myPrint(Item.." price",1000*price,textWidth,currency.."/1000 L");
		print(" ");
	end;


-- FERTILIZER

    idx = 5;
    Item = Commodity[idx];
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	price = randomPriceFactor * commodityDensity[idx]*commodityPricePer100kg[idx]/100;
	RN.sprayCost[1] = RN.sprayVolume[1] * price;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('FERTILIZER');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = commodityDensity[idx]/massScale;
    g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].previousHourPrice = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].startPricePerLiter = price;
	if isDebug then
		myPrint(Item.." fillTypeIndex",iFillType,textWidth," ");
		myPrintFloat(Item.." massPerLiter",commodityDensity[idx],textWidth," kg/L");
		myPrintFloat("RandomPriceFactor",randomPriceFactor,textWidth," ");
		myPrint(Item.." price",1000*price,textWidth,currency.."/1000 L");
		myPrint(Item.." price/ha",RN.sprayCost[1],textWidth,currency.."/ha");
		print(" ");
	end;


-- LIQUIDFERTILIZER

    idx = 6;
    Item = Commodity[idx];
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	price = randomPriceFactor * commodityDensity[idx]*commodityPricePer100kg[idx]/100;
	price = price / liqfertDilutionFactor;			-- 1 liter of fertilizer is diluted to 20 liters of solution
	RN.sprayCost[2] = RN.sprayVolume[2] * price;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('LIQUIDFERTILIZER');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = commodityDensity[idx]/massScale;
    g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].previousHourPrice = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].startPricePerLiter = price;
	if isDebug then
		myPrint(Item.." fillTypeIndex",iFillType,textWidth," ");
		myPrintFloat(Item.." massPerLiter",commodityDensity[idx],textWidth," kg/L");
		myPrintFloat("RandomPriceFactor",randomPriceFactor,textWidth," ");
		myPrint(Item.." (diluted) ",1000*price,textWidth,currency.."/1000 L");
		myPrint(Item.." price/ha",RN.sprayCost[2],textWidth,currency.."/ha");
		print(" ");
	end;


-- PIGFOOD

    idx = 7;
    Item = Commodity[idx];
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	price = randomPriceFactor * commodityDensity[idx]*commodityPricePer100kg[idx]/100;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('PIGFOOD');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = commodityDensity[idx]/massScale;
    g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].previousHourPrice = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].startPricePerLiter = price;
	if isDebug then
		myPrint(Item.." fillTypeIndex",iFillType,textWidth," ");
		myPrintFloat(Item.." massPerLiter",commodityDensity[idx],textWidth," kg/L");
		myPrintFloat("RandomPriceFactor",randomPriceFactor,textWidth," ");
		myPrint(Item.." price",1000*price,textWidth,currency.."/1000 L");
		print(" ");
	end;



-- FORAGE_MIXING

    idx = 8;
    Item = Commodity[idx];
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	price = randomPriceFactor * commodityDensity[idx]*commodityPricePer100kg[idx]/100;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('FORAGE_MIXING');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = commodityDensity[idx]/massScale;
    g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].previousHourPrice = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].startPricePerLiter = price;
	if isDebug then
		myPrint(Item.." fillTypeIndex",iFillType,textWidth," ");
		myPrintFloat(Item.." massPerLiter",commodityDensity[idx],textWidth," kg/L");
		myPrintFloat("RandomPriceFactor",randomPriceFactor,textWidth," ");
		myPrint(Item.." price",1000*price,textWidth,currency.."/1000 L");
		print(" ");
	end;



-- LIME

    idx = 9;
    Item = Commodity[idx];
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	price = randomPriceFactor * commodityDensity[idx]*commodityPricePer100kg[idx]/100;
	RN.sprayCost[6] = RN.sprayVolume[6] * price;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('LIME');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = commodityDensity[idx]/massScale;
    g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].previousHourPrice = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].startPricePerLiter = price;
	if isDebug then
		myPrint(Item.." fillTypeIndex",iFillType,textWidth," ");
		myPrintFloat(Item.." massPerLiter",commodityDensity[idx],textWidth," kg/L");
		myPrintFloat("RandomPriceFactor",randomPriceFactor,textWidth," ");
		myPrint(Item.." price",1000*price,textWidth,currency.."/1000 L");
		myPrint(Item.." price/ha",RN.sprayCost[6],textWidth,currency.."/ha");
		print(" ");
	end;


-- HERBICIDE

    idx = 10;
    Item = Commodity[idx];
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	price = randomPriceFactor * commodityDensity[idx]*commodityPricePer100kg[idx]/100;
	price = price / herbicideDilutionFactor;		-- 1 liter herbicide to 200 liter water
	RN.sprayCost[7] = RN.sprayVolume[7] * price;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('HERBICIDE');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = commodityDensity[idx]/massScale;
    g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].previousHourPrice = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].startPricePerLiter = price;
	if isDebug then
		myPrint(Item.." fillTypeIndex",iFillType,textWidth," ");
		myPrintFloat(Item.." massPerLiter",commodityDensity[idx],textWidth," kg/L");
		myPrintFloat("RandomPriceFactor",randomPriceFactor,textWidth," ");
		myPrint(Item.." (diluted) ",1000*price,textWidth,currency.."/1000 L");
		myPrint(Item.." price/ha",RN.sprayCost[7],textWidth,currency.."/ha");
		print(" ");
	end;

-- ROUNDBALE

    idx = 1;
    Item = roundBales[idx];
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	price = randomPriceFactor * roundBaleMass[idx] * roundBalePricePer100kg[idx]/(100 * roundBaleVolume[idx]);
    iFillType = g_fillTypeManager:getFillTypeIndexByName('ROUNDBALE');
    g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].previousHourPrice = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].startPricePerLiter = price;
	if isDebug then
		myPrint(Item.." fillTypeIndex",iFillType,textWidth," ");
		myPrintFloat("RandomPriceFactor",randomPriceFactor,textWidth," ");
		myPrint(Item.." price",1000*price,textWidth,currency.."/1000 L");
		print(" ");
	end;

-- ROUNDBALE_GRASS

    idx = 2;
    Item = roundBales[idx];
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	price = randomPriceFactor * roundBaleMass[idx] * roundBalePricePer100kg[idx]/(100 * roundBaleVolume[idx]);
    iFillType = g_fillTypeManager:getFillTypeIndexByName('ROUNDBALE_GRASS');
    g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].previousHourPrice = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].startPricePerLiter = price;
	if isDebug then
		myPrint(Item.." fillTypeIndex",iFillType,textWidth," ");
		myPrintFloat("RandomPriceFactor",randomPriceFactor,textWidth," ");
		myPrint(Item.." price",1000*price,textWidth,currency.."/1000 L");
		print(" ");
	end;

-- ROUNDBALE_DRYGRASS

    idx = 3;
    Item = roundBales[idx];
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	price = randomPriceFactor * roundBaleMass[idx] * roundBalePricePer100kg[idx]/(100 * roundBaleVolume[idx]);
    iFillType = g_fillTypeManager:getFillTypeIndexByName('ROUNDBALE_DRYGRASS');
    g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].previousHourPrice = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].startPricePerLiter = price;
	if isDebug then
		myPrint(Item.." fillTypeIndex",iFillType,textWidth," ");
		myPrintFloat("RandomPriceFactor",randomPriceFactor,textWidth," ");
		myPrint(Item.." price",1000*price,textWidth,currency.."/1000 L");
		print(" ");
	end;

-- ROUNDBALE_WHEAT

    idx = 4;
    Item = roundBales[idx];
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	price = randomPriceFactor * roundBaleMass[idx] * roundBalePricePer100kg[idx]/(100 * roundBaleVolume[idx]);
    iFillType = g_fillTypeManager:getFillTypeIndexByName('ROUNDBALE_WHEAT');
    g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].previousHourPrice = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].startPricePerLiter = price;
	if isDebug then
		myPrint(Item.." fillTypeIndex",iFillType,textWidth," ");
		myPrintFloat("RandomPriceFactor",randomPriceFactor,textWidth," ");
		myPrint(Item.." price",1000*price,textWidth,currency.."/1000 L");
		print(" ");
	end;

-- ROUNDBALE_BARLEY

    idx = 5;
    Item = roundBales[idx];
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	price = randomPriceFactor * roundBaleMass[idx] * roundBalePricePer100kg[idx]/(100 * roundBaleVolume[idx]);
    iFillType = g_fillTypeManager:getFillTypeIndexByName('ROUNDBALE_BARLEY');
    g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].previousHourPrice = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].startPricePerLiter = price;
	if isDebug then
		myPrint(Item.." fillTypeIndex",iFillType,textWidth," ");
		myPrintFloat("RandomPriceFactor",randomPriceFactor,textWidth," ");
		myPrint(Item.." price",1000*price,textWidth,currency.."/1000 L");
		print(" ");
	end;


-- SQUAREBALE

    idx = 1;
    Item = squareBales[idx];
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	price = randomPriceFactor * squareBaleMass[idx] * squareBalePricePer100kg[idx]/(100 * squareBaleVolume[idx]);
    iFillType = g_fillTypeManager:getFillTypeIndexByName('SQUAREBALE');
    g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].previousHourPrice = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].startPricePerLiter = price;
	if isDebug then
		myPrint(Item.." fillTypeIndex",iFillType,textWidth," ");
		myPrintFloat("RandomPriceFactor",randomPriceFactor,textWidth," ");
		myPrint(Item.." price",1000*price,textWidth,currency.."/1000 L");
		print(" ");
	end;

-- SQUAREBALE_WHEAT

    idx = 2;
    Item = squareBales[idx];
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	price = randomPriceFactor * squareBaleMass[idx] * squareBalePricePer100kg[idx]/(100 * squareBaleVolume[idx]);
    iFillType = g_fillTypeManager:getFillTypeIndexByName('SQUAREBALE_WHEAT');
    g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].previousHourPrice = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].startPricePerLiter = price;
	if isDebug then
		myPrint(Item.." fillTypeIndex",iFillType,textWidth," ");
		myPrintFloat("RandomPriceFactor",randomPriceFactor,textWidth," ");
		myPrint(Item.." price",1000*price,textWidth,currency.."/1000 L");
		print(" ");
	end;

-- SQUAREBALE_BARLEY

    idx = 3;
    Item = squareBales[idx];
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	price = randomPriceFactor * squareBaleMass[idx] * squareBalePricePer100kg[idx]/(100 * squareBaleVolume[idx]);
    iFillType = g_fillTypeManager:getFillTypeIndexByName('SQUAREBALE_BARLEY');
    g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].previousHourPrice = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].startPricePerLiter = price;
	if isDebug then
		myPrint(Item.." fillTypeIndex",iFillType,textWidth," ");
		myPrintFloat("RandomPriceFactor",randomPriceFactor,textWidth," ");
		myPrint(Item.." price",1000*price,textWidth,currency.."/1000 L");
		print(" ");
	end;

-- MANURE

    idx = 1;
    Item = organicMaterials[idx];
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	price = randomPriceFactor * organicPrices[idx];
	RN.sprayCost[3] = RN.sprayVolume[3] * price;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('MANURE');
    g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].previousHourPrice = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].startPricePerLiter = price;
	if isDebug then
		myPrint(Item.." fillTypeIndex",iFillType,textWidth," ");
		myPrintFloat("RandomPriceFactor",randomPriceFactor,textWidth," ");
		myPrint(Item.." price",1000*price,textWidth,currency.."/1000 L");
		myPrint(Item.." price/ha",RN.sprayCost[3],textWidth,currency.."/ha");
		print(" ");
	end;

-- LIQUIDMANURE

    idx = 2;
    Item = organicMaterials[idx];
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	price = randomPriceFactor * organicPrices[idx];
	RN.sprayCost[4] = RN.sprayVolume[4] * price;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('LIQUIDMANURE');
    g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].previousHourPrice = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].startPricePerLiter = price;
	if isDebug then
		myPrint(Item.." fillTypeIndex",iFillType,textWidth," ");
		myPrintFloat("RandomPriceFactor",randomPriceFactor,textWidth," ");
		myPrint(Item.." price",1000*price,textWidth,currency.."/1000 L");
		myPrint(Item.." price/ha",RN.sprayCost[4],textWidth,currency.."/ha");
		print(" ");
	end;

-- DIGESTATE

    idx = 3;
    Item = organicMaterials[idx];
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	price = randomPriceFactor * organicPrices[idx];
	RN.sprayCost[5] = RN.sprayVolume[5] * price;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('DIGESTATE');
    g_currentMission.fillTypeManager.fillTypes[iFillType].pricePerLiter = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].previousHourPrice = price;
	g_currentMission.fillTypeManager.fillTypes[iFillType].startPricePerLiter = price;
	if isDebug then
		myPrint(Item.." fillTypeIndex",iFillType,textWidth," ");
		myPrintFloat("RandomPriceFactor",randomPriceFactor,textWidth," ");
		myPrint(Item.." price",1000*price,textWidth,currency.."/1000 L");
		myPrint(Item.." price/ha",RN.sprayCost[5],textWidth,currency.."/ha");
		print(" ");
	end;

-- Store objects

-- Bigbag pigfood

	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	storeObject = g_storeManager:getItemByIndex(407);
	storeObject.price = randomPriceFactor * RN.bigbagVolume[1] * RN.commodityDensity[7] * RN.commodityPricePer100kg[7]/100;

-- Bigbag horsefood
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	storeObject = g_storeManager:getItemByIndex(408);
	storeObject.price = randomPriceFactor * RN.bigbagVolume[2] * RN.commodityDensity[7] * RN.commodityPricePer100kg[7]/100;

-- Bigbag chickenfood
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	storeObject = g_storeManager:getItemByIndex(409);
	storeObject.price = randomPriceFactor * RN.bigbagVolume[3] * RN.commodityDensity[7] * RN.commodityPricePer100kg[7]/100;

-- Bigbag seeds
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	storeObject = g_storeManager:getItemByIndex(410);
	storeObject.price = randomPriceFactor * RN.bigbagVolume[4] * RN.commodityDensity[3] * RN.commodityPricePer100kg[3]/100;

-- Bigbag fertilizer
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	storeObject = g_storeManager:getItemByIndex(411);
	storeObject.price = randomPriceFactor * RN.bigbagVolume[5] * RN.commodityDensity[5] * RN.commodityPricePer100kg[5]/100;

-- Bigbag lime
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	storeObject = g_storeManager:getItemByIndex(412);
	storeObject.price = randomPriceFactor * RN.bigbagVolume[6] * RN.commodityDensity[9] * RN.commodityPricePer100kg[9]/100;

-- Liquid fertilizer pallet
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	storeObject = g_storeManager:getItemByIndex(413);
	storeObject.price = randomPriceFactor * RN.storePalletVolume[1] * RN.commodityDensity[6] * RN.commodityPricePer100kg[6]/(20 * 100);

-- Herbicide pallet
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	storeObject = g_storeManager:getItemByIndex(414);
	storeObject.price = randomPriceFactor * RN.storePalletVolume[2] * RN.commodityDensity[10] * RN.commodityPricePer100kg[10]/(200 * 100);

-- Tree saplings
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	storeObject = g_storeManager:getItemByIndex(415);
	storeObject.price = randomPriceFactor * RN.storePalletVolume[3] * RN.treeSaplingsUnitPrice;

-- Poplar saplings
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	storeObject = g_storeManager:getItemByIndex(416);
	storeObject.price = randomPriceFactor * RN.storePalletVolume[4] * RN.seedDensity[12] * RN.seedPricePer100kg[12]/100;

-- Sugarcane
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	storeObject = g_storeManager:getItemByIndex(417);
	storeObject.price = randomPriceFactor * RN.storePalletVolume[5] * RN.seedDensity[15] * RN.seedPricePer100kg[15]/100;

-- Square bale straw
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	storeObject = g_storeManager:getItemByIndex(421);
	storeObject.price = randomPriceFactor * RN.storeBalesMass[1] * RN.storeBalesPricePer100kg[1]/100;

-- Square bale hay
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	storeObject = g_storeManager:getItemByIndex(422);
	storeObject.price = randomPriceFactor * RN.storeBalesMass[2] * RN.storeBalesPricePer100kg[2]/100;

-- Round bale silage
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	storeObject = g_storeManager:getItemByIndex(423);
	storeObject.price = randomPriceFactor * RN.storeBalesMass[3] * RN.storeBalesPricePer100kg[3]/100;

-- Round bale straw
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	storeObject = g_storeManager:getItemByIndex(424);
	storeObject.price = randomPriceFactor * RN.storeBalesMass[4] * RN.storeBalesPricePer100kg[4]/100;

-- Round bale hay
	randomPriceFactor = 1 + randomness * RN.commodityPriceVariation * (math.random()-0.5)/100;
	storeObject = g_storeManager:getItemByIndex(425);
	storeObject.price = randomPriceFactor * RN.storeBalesMass[5] * RN.storeBalesPricePer100kg[5]/100;

	if isDebug then print("\nRealNumbersCommodityPrices Debug Report End \n") end;
end;


function RealNumbersCommodityPrices:keyEvent(unicode, sym, modifier, isDown)
end;

function RealNumbersCommodityPrices:update(dt)
	if g_fieldManager.fields ~= nil then
		g_farmlandManager.pricePerHa = RN.pricePerHa;   -- set new value value
		local randomness = 1;							-- set to zero for no fluctuation, set to 1 for +/- 5% fluctuation
		for k,field in pairs(g_fieldManager.fields) do
			local farmland = field.farmland;
			local farmlandArea = farmland.areaInHa;
			local randomFieldPriceFactor = 1 + randomness * 1 * (math.random()-0.5)/10;
			farmland.price = randomFieldPriceFactor * RN.fieldPriceList[k] * farmlandArea;
		end;
	end;

	if g_farmManager.farms ~= nil then
		for k,farm in pairs(g_farmManager.farms) do
			farm.loanAnnualInterestRate = RN.annualLoanInterest;
			farm.loanMax = RN.loanMax;
		end;
	end;

	if g_storeManager.items ~= nil then
		for k,items in pairs(g_storeManager.items) do
			if items.allowLeasing then
				items.runningLeasingFactor  = RN.DEFAULT_RUNNING_LEASING_FACTOR;
			end;
		end;
	end;
	
	local gameLevel = g_currentMission.missionInfo.economicDifficulty;
	g_currentMission.economyManager.PRICE_MULTIPLIER[gameLevel] = RN.gameLevel.sellPriceMultiplier[gameLevel];
	g_currentMission.economyManager.COST_MULTIPLIER[gameLevel] = RN.gameLevel.buyPriceMultiplier[gameLevel];
	g_currentMission.economyManager.DIRECT_SELL_MULTIPLIER = RN.DIRECT_SELL_MULTIPLIER;

-- the following asignments are stored, but apparently not used as the leasing cost is not reduced.
	
	g_currentMission.economyManager.DEFAULT_RUNNING_LEASING_FACTOR = RN.DEFAULT_RUNNING_LEASING_FACTOR;
	g_currentMission.economyManager.PER_DAY_LEASING_FACTOR = RN.PER_DAY_LEASING_FACTOR;
	g_currentMission.economyManager.DEFAULT_LEASING_DEPOSIT_FACTOR = RN.DEFAULT_LEASING_DEPOSIT_FACTOR;

-- don't know how to fix it.
	
end;
function RealNumbersCommodityPrices:draw()
end;
function RealNumbersCommodityPrices:deleteMap()
end;
function RealNumbersCommodityPrices:mouseEvent(posX, posY, isDown, isUp, button)
end;

addModEventListener(RealNumbersCommodityPrices);
