--
-- Mod: FS19_RealLifeNumbers by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- Script: RealNumbersCropYield by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- e-mail: kaj-aage.henneberg@outlook.dk
-- All rights reserved.
-- Version 1.0.0.0
-- 30.11.2018 - 20.03.2019

-- Names for FillTypes

-- WHEAT BARLEY OAT CANOLA SUNFLOWER SOYBEAN MAIZE POTATO SUGARBEET OILSEEDRADISH
-- MANURE SEEDS FORAGE FORAGE_MIXING CHAFF WOODCHIPS SILAGE
-- STRAW GRASS_WINDROW DRYGRASS_WINDROW SUGARCANE FERTILIZER PIGFOOD LIME
-- WOOL TREESAPLINGS EGG
-- MILK FUEL WATER DIESEL DEF
-- LIQUIDFERTILIZER HERBICIDE
-- CHAFF GRASS_WINDROW
-- LIQUIDMANURE DIGESTATE
-- COW_TYPE_BRAHMAN_GREY COW_TYPE_BRAHMAN_LIGHT_BROWN COW_TYPE_BRAHMAN_WHITE
-- COW_TYPE_BRAHMAN_BROWN COW_TYPE_BROWN COW_TYPE_BROWN_WHITE COW_TYPE_BLACK
-- COW_TYPE_BLACK_WHITE
-- SHEEP_TYPE_WHITE SHEEP_TYPE_BROWN SHEEP_TYPE_BLACK_WHITE SHEEP_TYPE_BLACK
-- PIG_TYPE_RED PIG_TYPE_WHITE PIG_TYPE_BLACK_WHITE PIG_TYPE_BLACK
-- CHICKEN_TYPE_BLACK CHICKEN_TYPE_WHITE CHICKEN_TYPE_BROWN CHICKEN_TYPE_ROOSTER

-- FruitType indices

	-- WHEAT :: 1, BARLEY :: 2, OAT :: 3, COTTON :: 4, CANOLA :: 5, SUNFLOWER :: 6,
	-- SOYBEAN :: 7, MAIZE :: 8, POTATO :: 9, SUGARBEET :: 10, OILSEEDRADISH :: 11,
	-- POPLAR :: 12, GRASS :: 13, DRYGRASS :: 14, SUGARCANE :: 15, WEED :: 16


--[[
Crop mass density

	Crop mass density (kg/Liter) varies as a function of moisture content. Different sources will therefore
	provide slightly different numbers. The majority of the numbers listed above are taken from
	The agricultural service of the province of Alberta, Canada.

	Be aware that Farmingsimulator requires density numbers in (metric) Tonne/Liter. Hence the numbers listed
	above is divided by 1000 when assigning the value to massPerLiter.

	Crop mass density is considered independent of geographical location.

Crop Yield

	Crop yield is assumed to be dependent on geographical location.

	Numbers for crop yield are given in a number of different units.
	EUROSTATS provides yield in	multiples of 100 kg/hectare or Tonne/hectare
	US National Agricultural Statistics Service uses lb/acre, bushels/acre, cwt/acre, ton/acre.
	Conversion to Liter/ha is required and from there to Liters/square meters.
	The following conversion numbers are used:

		1 bushel = 35.239072 Liters
		1 acre = 0.404685642 hectares
		1 CWT = 45.35924 kg

	All density numbers in the cropYield list is provided in multiples of 100kg/ha. The cropDensity
	values are used to convert cropYield numbers to Liters/square meter.

	The base yield X set in the game must be calculated from the national/regional yield Y.

	The help suggests that application of lime and fertilizer will add 15% and 25% respectively. Thus,
	one application of lime and two applications of fertilizer should give 65% increase relative to base
	yield, suggesting that X + 0.65*X = Y, that is X = Y/1.65.

	However, tests suggest that this is not how it works. It appear that base yield is the minimum yield, you
	can get if you neglect to plough, remove weed, lime and fertilizer. Doing all this chores will add 100 %
	to the base yield. It seems therefore that X + 1*X = 2 X = Y, or X = Y/2. For this reason the parameter
	gameCorrection = 1/2, not 1/1.65 as one would expect.

	To include natural variations in crop yield, the yield for each crop is scaled by a randomness factor.
	The randomness factor varies within +/- two standard deviations for the given crop. Some crops
	have a small yield fluctuation, others a larger fluctuation, all dependng on the crop and geographical
	location. The randomness factor changes at each game load. It is the value at harvet, that determines the
	obtained yield. If the harvest of a field is divided between several gameplays, different yield setting will
	be encountered for each gameplay. This may be taken to represent yield variation across the field.

	Random variation of the yield can be turned off by different values of

	randomness = 0  => randomness is off
	randomness = 1  => randomness is on

	If you want to calibrate yield, this is better done without randomness in the yield parameter. After
	calibration, randomness can be turned back on.

Windrow Yield

	Windrow yield for grain crops is about 80% of grain yield measured in kg/ha.
	Straw density is set to 0.13 kg/Liter

	Look in the log file for printed output from this script.

]]--

RealNumbersCropYield = {};


function RealNumbersCropYield:loadMap(name)
    local textWidth = 40; 					-- text width in log file
    local iFillType = 0;
	local iFruitType = 0;
	local Crop = "Empty";
	local idx = 0;
	local bonusYield = 0;
	local baseYield = 0;
	local bonusWindrow = 0;
	local gameWindrow = 0;
	local lossWindrow = 0.05;
	local seedRate = 0;
	local price = 0;
	local bonusScale = 1.0;						-- for adding extra yield for perfect crop management
	local massScale = 1000;						-- FS uses Tonne/Liter for mass density
	local gameCorrection = RN.fieldWorkbaseYieldAdjustment;	-- scale factor to obtain base yield from bonus yield

	local randomYieldFactor = 1;				-- default value used if reading seed number fails
	local randomness = 1;						-- Set to 1 to turn randomness on, 0 to turn off
	local playTime = nil;
	local noPloughLoss = 0;
	local noWeedingLoss = 0;
	local LimeBonus = 0;
	local FertilizerBonus = 0;
	local grain = {};
	local rowgrain = {};
	local sugarbeet = {};
	local potato = {};
	local isDebug = RN.isDebug;  						-- Change to False when no printing to log file is desired
	local isDebug2 = false;

	local gameLevelYieldCorrection = RN.gameLevel.cropYieldCorrection;
	local Fruits = RN.Fruits;							-- local copy of global variable
	local cropDensity = RN.cropDensity;					-- local copy of global variable
	local cropYield = RN.cropYield;						-- local copy of global variable
	local yieldRange = RN.yieldRange;					-- local copy of global variable
	local strawDensity = RN.roundBaleMass[4]/RN.roundBaleVolume[4];

-- FruitType indices

	-- WHEAT :: 1, BARLEY :: 2, OAT :: 3, COTTON :: 4, CANOLA :: 5, SUNFLOWER :: 6,
	-- SOYBEAN :: 7, MAIZE :: 8, POTATO :: 9, SUGARBEET :: 10, OILSEEDRADISH :: 11,
	-- POPLAR :: 12, GRASS :: 13, DRYGRASS :: 14, SUGARCANE :: 15, WEED :: 16

	print("FS19_RealLifeNumbers: RealNumbersCropYield active")



	if isDebug then print("\nRealNumbersCropYield Debug Report Start \n") end;

	grain.headerwidth = RN.grainHeaderwidth; 						-- m
	grain.workingspeed = RN.grainWorkingspeed;						-- kph
	grain.AreaPerSec = grain.headerwidth * grain.workingspeed/3.6;
	grain.hectarePer30Sec = 30 * grain.AreaPerSec/10000;

	rowgrain.headerwidth = RN.rowgrainHeaderwidth; 						-- m
	rowgrain.workingspeed = RN.rowgrainWorkingspeed;						-- kph
	rowgrain.AreaPerSec = rowgrain.headerwidth * rowgrain.workingspeed/3.6;
	rowgrain.hectarePer30Sec = 30 * rowgrain.AreaPerSec/10000;

	potato.headerwidth = RN.potatoHeaderwidth; 					-- m
	potato.workingspeed = RN.potatoWorkingspeed;					-- kph
	potato.AreaPerSec = potato.headerwidth * potato.workingspeed/3.6;
	potato.hectarePer30Sec = 30 * potato.AreaPerSec/10000;

	sugarbeet.headerwidth = RN.sugarbeetHeaderwidth; 					-- m
	sugarbeet.workingspeed = RN.sugarbeetWorkingspeed;					-- kph
	sugarbeet.AreaPerSec = sugarbeet.headerwidth * sugarbeet.workingspeed/3.6;
	sugarbeet.hectarePer30Sec = 30 * sugarbeet.AreaPerSec/10000;


	local savegameDirectory = g_currentMission.missionInfo.savegameDirectory;	-- works except when a new map is opened the first time
	if savegameDirectory ~= nil then
		local careerFile = savegameDirectory.."/careerSavegame.xml";
		if isDebug2 then print("careerFile : "..tostring(careerFile)); end;
		local xmlFile = loadXMLFile("careerFile", careerFile);
		local key = string.format("careerSavegame.statistics");
		if not hasXMLProperty(xmlFile, "careerSavegame") then
			if isDebug2 then print("Does not have XMLproperty"); end;
		end;
		playTime = 1000 * getXMLFloat(xmlFile, "careerSavegame.statistics.playTime");
	end;

	if playTime == nil then
		if isDebug2 then print("playTime is nil") end;
		randomness = 0;
	else
		local randomSeednumber = math.randomseed(playTime);
		if isDebug2 then print("playTime is "..playTime/1000); end;
		if isDebug2 then print("random number is "..math.random()); end;
	end;

-- WHEAT

    idx = 1;
    Crop = Fruits[idx];
	randomYieldFactor = 1 + randomness * 2*(math.random()-0.5) * yieldRange[idx];
	bonusYield = bonusScale * randomYieldFactor * cropYield[idx]*100/(cropDensity[idx]*10000); --Real life yield in liters/sqm
	baseYield =  gameCorrection *  bonusYield;
	bonusWindrow = (1-lossWindrow) * bonusScale * randomYieldFactor * 0.8 * cropYield[idx]*100/(strawDensity*10000);
	RN.wheatWindrowYieldLitersPerHa = 10000 * bonusWindrow;
	RN.wheatWindrowYieldkgPerHa = (1-lossWindrow) * bonusScale * randomYieldFactor * 0.8 * cropYield[idx] * 100;
	gameWindrow =  gameCorrection * bonusWindrow;
	iFillType = g_fillTypeManager:getFillTypeIndexByName(Fruits[idx]); 	--Smarter use, but requires Fruits to be a local variable
--    iFillType = g_fillTypeManager:getFillTypeIndexByName('WHEAT');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = cropDensity[idx]/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].literPerSqm = gameLevelYieldCorrection * baseYield;
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowLiterPerSqm = gameLevelYieldCorrection * gameWindrow;
	noPloughLoss = 10000 * baseYield * 0.15;				-- 15% loss
	noWeedingLoss = 10000 * baseYield * 0.2;				-- 20% loss
	LimeBonus = 10000 * baseYield * 0.15;					-- 15% gain
	FertilizerBonus = 10000 * baseYield * 0.25;				-- 25% gain

	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." massPerLiter",cropDensity[idx],textWidth," kg/L");
		myPrintFloat("Yield randomness scaling",randomYieldFactor,textWidth," ");
		myPrintFloat(Crop.." base yield",baseYield,textWidth," L/sqm");
		myPrint(Crop.." base yield",10000*baseYield,textWidth," L/Ha");
		myPrint(Crop.." no-plough loss",noPloughLoss,textWidth," L/Ha");
		myPrint(Crop.." no-weeding loss",noWeedingLoss,textWidth," L/Ha");
		myPrint(Crop.." liming gain",LimeBonus,textWidth," L/Ha");
		myPrint(Crop.." 1x fertilizing gain",FertilizerBonus,textWidth," L/Ha");
		myPrint(Crop.." max yield",10000*bonusYield,textWidth," L/Ha");
		myPrintFloat(Crop.." hectar per 30 sec",grain.hectarePer30Sec,textWidth," Ha");
		myPrint(Crop.." Liters per 30 sec",10000*bonusYield * grain.hectarePer30Sec,textWidth," L/30 sec");
		myPrint(Crop.." windrowLiterPerHa",10000* bonusWindrow,textWidth," L/Ha\n");
	end;

--BARLEY

    idx = 2;
    Crop = Fruits[idx];
	randomYieldFactor = 1 + randomness * 2*(math.random()-0.5) * yieldRange[idx];
	bonusYield = bonusScale * randomYieldFactor * cropYield[idx]*100/(cropDensity[idx]*10000); --Real life yield in liters/sqm
	baseYield = gameCorrection *  bonusYield;
	bonusWindrow = (1-lossWindrow) * bonusScale * randomYieldFactor * 0.8 * cropYield[idx]*100/(strawDensity*10000);
	RN.barleyWindrowYieldLitersPerHa = 10000 * bonusWindrow;
	RN.barleyWindrowYieldkgPerHa = (1-lossWindrow) * bonusScale * randomYieldFactor * 0.8 * cropYield[idx] * 100;
	gameWindrow =  gameCorrection * bonusWindrow;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('BARLEY');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = cropDensity[idx]/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].literPerSqm =  gameLevelYieldCorrection * baseYield;
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowLiterPerSqm = gameLevelYieldCorrection * gameWindrow;
	noPloughLoss = 10000 * baseYield * 0.15;				-- 15% loss
	noWeedingLoss = 10000 * baseYield * 0.2;				-- 20% loss
	LimeBonus = 10000 * baseYield * 0.15;					-- 15% gain
	FertilizerBonus = 10000 * baseYield * 0.25;				-- 25% gain

	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." massPerLiter",cropDensity[idx],textWidth," kg/L");
		myPrintFloat("Yield randomness scaling",randomYieldFactor,textWidth," ");
		myPrintFloat(Crop.." base yield",baseYield,textWidth," L/sqm");
		myPrint(Crop.." base yield",10000*baseYield,textWidth," L/Ha");
		myPrint(Crop.." no-plough loss",noPloughLoss,textWidth," L/Ha");
		myPrint(Crop.." no-weeding loss",noWeedingLoss,textWidth," L/Ha");
		myPrint(Crop.." liming gain",LimeBonus,textWidth," L/Ha");
		myPrint(Crop.." 1x fertilizing gain",FertilizerBonus,textWidth," L/Ha");
		myPrint(Crop.." max yield",10000*bonusYield,textWidth," L/Ha");
		myPrintFloat(Crop.." hectar per 30 sec",grain.hectarePer30Sec,textWidth," Ha");
		myPrint(Crop.." Liters per 30 sec",10000*bonusYield * grain.hectarePer30Sec,textWidth," L/30 sec");
		myPrint(Crop.." windrowLiterPerHa",10000* bonusWindrow,textWidth," L/Ha\n");
	end;

--OAT

	idx = 3;
    Crop = Fruits[idx];
	randomYieldFactor = 1 + randomness * 2*(math.random()-0.5) * yieldRange[idx];
	bonusYield = bonusScale * randomYieldFactor * cropYield[idx]*100/(cropDensity[idx]*10000); --Real life yield in liters/sqm
	baseYield = gameCorrection *  bonusYield;
	bonusWindrow = (1-lossWindrow) * bonusScale * randomYieldFactor * 0.8 * cropYield[idx]*100/(strawDensity*10000);
	RN.oatWindrowYieldLitersPerHa = 10000 * bonusWindrow;
	RN.oatWindrowYieldkgPerHa = (1-lossWindrow) * bonusScale * randomYieldFactor * 0.8 * cropYield[idx] * 100;
	gameWindrow =  gameCorrection * bonusWindrow;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('OAT');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = cropDensity[idx]/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].literPerSqm =  gameLevelYieldCorrection * baseYield;
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowLiterPerSqm = gameLevelYieldCorrection * gameWindrow;
	noPloughLoss = 10000 * baseYield * 0.15;				-- 15% loss
	noWeedingLoss = 10000 * baseYield * 0.2;				-- 20% loss
	LimeBonus = 10000 * baseYield * 0.15;					-- 15% gain
	FertilizerBonus = 10000 * baseYield * 0.25;				-- 25% gain
	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." massPerLiter",cropDensity[idx],textWidth," kg/L");
		myPrintFloat("Yield randomness scaling",randomYieldFactor,textWidth," ");
		myPrintFloat(Crop.." base yield",baseYield,textWidth," L/sqm");
		myPrint(Crop.." base yield",10000*baseYield,textWidth," L/Ha");
		myPrint(Crop.." no-plough loss",noPloughLoss,textWidth," L/Ha");
		myPrint(Crop.." no-weeding loss",noWeedingLoss,textWidth," L/Ha");
		myPrint(Crop.." liming gain",LimeBonus,textWidth," L/Ha");
		myPrint(Crop.." 1x fertilizing gain",FertilizerBonus,textWidth," L/Ha");
		myPrint(Crop.." max yield",10000*bonusYield,textWidth," L/Ha");
		myPrintFloat(Crop.." hectar per 30 sec",grain.hectarePer30Sec,textWidth," Ha");
		myPrint(Crop.." Liters per 30 sec",10000*bonusYield * grain.hectarePer30Sec,textWidth," L/30 sec");
		myPrint(Crop.." windrowLiterPerHa",10000* bonusWindrow,textWidth," L/Ha\n");
	end;

--COTTON

	idx = 4;
    Crop = Fruits[idx];
	randomYieldFactor = 1 + randomness * 2*(math.random()-0.5) * yieldRange[idx];
	bonusYield = bonusScale * randomYieldFactor * cropYield[idx]*100/(cropDensity[idx]*10000); --Real life yield in liters/sqm
	baseYield = gameCorrection * bonusYield;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('COTTON');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = cropDensity[idx]/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].literPerSqm =  gameLevelYieldCorrection * baseYield;
	noPloughLoss = 10000 * baseYield * 0.15;				-- 15% loss
	noWeedingLoss = 10000 * baseYield * 0.2;				-- 20% loss
	LimeBonus = 10000 * baseYield * 0.15;					-- 15% gain
	FertilizerBonus = 10000 * baseYield * 0.25;				-- 25% gain
	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." massPerLiter",cropDensity[idx],textWidth," kg/L");
		myPrintFloat("Yield randomness scaling",randomYieldFactor,textWidth," ");
		myPrintFloat(Crop.." base yield",baseYield,textWidth," L/sqm");
		myPrint(Crop.." base yield",10000*baseYield,textWidth," L/Ha");
		myPrint(Crop.." no-plough loss",noPloughLoss,textWidth," L/Ha");
		myPrint(Crop.." no-weeding loss",noWeedingLoss,textWidth," L/Ha");
		myPrint(Crop.." liming gain",LimeBonus,textWidth," L/Ha");
		myPrint(Crop.." 1x fertilizing gain",FertilizerBonus,textWidth," L/Ha");
		myPrint(Crop.." max yield",10000*bonusYield,textWidth," L/Ha\n");
	end;


--CANOLA

	idx = 5;
    Crop = Fruits[idx];
	randomYieldFactor = 1 + randomness * 2*(math.random()-0.5) * yieldRange[idx];
	bonusYield = bonusScale * randomYieldFactor * cropYield[idx]*100/(cropDensity[idx]*10000); --Real life yield in liters/sqm
	baseYield = gameCorrection * bonusYield;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('CANOLA');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = cropDensity[idx]/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].literPerSqm =  gameLevelYieldCorrection * baseYield;
	noPloughLoss = 10000 * baseYield * 0.15;				-- 15% loss
	noWeedingLoss = 10000 * baseYield * 0.2;				-- 20% loss
	LimeBonus = 10000 * baseYield * 0.15;					-- 15% gain
	FertilizerBonus = 10000 * baseYield * 0.25;				-- 25% gain
	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." massPerLiter",cropDensity[idx],textWidth," kg/L");
		myPrintFloat("Yield randomness scaling",randomYieldFactor,textWidth," ");
		myPrintFloat(Crop.." base yield",baseYield,textWidth," L/sqm");
		myPrint(Crop.." base yield",10000*baseYield,textWidth," L/Ha");
		myPrint(Crop.." no-plough loss",noPloughLoss,textWidth," L/Ha");
		myPrint(Crop.." no-weeding loss",noWeedingLoss,textWidth," L/Ha");
		myPrint(Crop.." liming gain",LimeBonus,textWidth," L/Ha");
		myPrint(Crop.." 1x fertilizing gain",FertilizerBonus,textWidth," L/Ha");
		myPrint(Crop.." max yield",10000*bonusYield,textWidth," L/Ha");
		myPrintFloat(Crop.." hectar per 30 sec",grain.hectarePer30Sec,textWidth," Ha");
		myPrint(Crop.." Liters per 30 sec",10000*bonusYield * grain.hectarePer30Sec,textWidth," L/30 sec\n");
	end;

--SUNFLOWER

	idx = 6;
    Crop = Fruits[idx];
	randomYieldFactor = 1 + randomness * 2*(math.random()-0.5) * yieldRange[idx];
	bonusYield = bonusScale * randomYieldFactor * cropYield[idx]*100/(cropDensity[idx]*10000); --Real life yield in liters/sqm
	baseYield = gameCorrection * bonusYield;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('SUNFLOWER');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = cropDensity[idx]/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].literPerSqm =  gameLevelYieldCorrection * baseYield;
	noPloughLoss = 10000 * baseYield * 0.15;				-- 15% loss
	noWeedingLoss = 10000 * baseYield * 0.2;				-- 20% loss
	LimeBonus = 10000 * baseYield * 0.15;					-- 15% gain
	FertilizerBonus = 10000 * baseYield * 0.25;				-- 25% gain
	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." massPerLiter",cropDensity[idx],textWidth," kg/L");
		myPrintFloat("Yield randomness scaling",randomYieldFactor,textWidth," ");
		myPrintFloat(Crop.." base yield",baseYield,textWidth," L/sqm");
		myPrint(Crop.." base yield",10000*baseYield,textWidth," L/Ha");
		myPrint(Crop.." no-plough loss",noPloughLoss,textWidth," L/Ha");
		myPrint(Crop.." no-weeding loss",noWeedingLoss,textWidth," L/Ha");
		myPrint(Crop.." liming gain",LimeBonus,textWidth," L/Ha");
		myPrint(Crop.." 1x fertilizing gain",FertilizerBonus,textWidth," L/Ha");
		myPrint(Crop.." max yield",10000*bonusYield,textWidth," L/Ha");
		myPrintFloat(Crop.." hectar per 30 sec",rowgrain.hectarePer30Sec,textWidth," Ha");
		myPrint(Crop.." Liters per 30 sec",10000*bonusYield * rowgrain.hectarePer30Sec,textWidth," L/30 sec\n");
	end;


--SOYBEAN

	idx = 7;
    Crop = Fruits[idx];
	randomYieldFactor = 1 + randomness * 2*(math.random()-0.5) * yieldRange[idx];
	bonusYield = bonusScale * randomYieldFactor * cropYield[idx]*100/(cropDensity[idx]*10000); --Real life yield in liters/sqm
	baseYield = gameCorrection * bonusYield;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('SOYBEAN');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = cropDensity[idx]/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].literPerSqm =  gameLevelYieldCorrection * baseYield;
	noPloughLoss = 10000 * baseYield * 0.15;				-- 15% loss
	noWeedingLoss = 10000 * baseYield * 0.2;				-- 20% loss
	LimeBonus = 10000 * baseYield * 0.15;					-- 15% gain
	FertilizerBonus = 10000 * baseYield * 0.25;				-- 25% gain
	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." massPerLiter",cropDensity[idx],textWidth," kg/L");
		myPrintFloat("Yield randomness scaling",randomYieldFactor,textWidth," ");
		myPrintFloat(Crop.." base yield",baseYield,textWidth," L/sqm");
		myPrint(Crop.." base yield",10000*baseYield,textWidth," L/Ha");
		myPrint(Crop.." no-plough loss",noPloughLoss,textWidth," L/Ha");
		myPrint(Crop.." no-weeding loss",noWeedingLoss,textWidth," L/Ha");
		myPrint(Crop.." liming gain",LimeBonus,textWidth," L/Ha");
		myPrint(Crop.." 1x fertilizing gain",FertilizerBonus,textWidth," L/Ha");
		myPrint(Crop.." max yield",10000*bonusYield,textWidth," L/Ha");
		myPrintFloat(Crop.." hectar per 30 sec",grain.hectarePer30Sec,textWidth," Ha");
		myPrint(Crop.." Liters per 30 sec",10000*bonusYield * grain.hectarePer30Sec,textWidth," L/30 sec\n");
	end;


--MAIZE

	idx = 8;
    Crop = Fruits[idx];
	randomYieldFactor = 1 + randomness * 2*(math.random()-0.5) * yieldRange[idx];
	bonusYield = bonusScale * randomYieldFactor * cropYield[idx]*100/(cropDensity[idx]*10000); --Real life yield in liters/sqm
	baseYield = gameCorrection * bonusYield;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('MAIZE');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = cropDensity[idx]/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].literPerSqm =  gameLevelYieldCorrection * baseYield;
	noPloughLoss = 10000 * baseYield * 0.15;				-- 15% loss
	noWeedingLoss = 10000 * baseYield * 0.2;				-- 20% loss
	LimeBonus = 10000 * baseYield * 0.15;					-- 15% gain
	FertilizerBonus = 10000 * baseYield * 0.25;				-- 25% gain
	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." massPerLiter",cropDensity[idx],textWidth," kg/L");
		myPrintFloat("Yield randomness scaling",randomYieldFactor,textWidth," ");
		myPrintFloat(Crop.." base yield",baseYield,textWidth," L/sqm");
		myPrint(Crop.." base yield",10000*baseYield,textWidth," L/Ha");
		myPrint(Crop.." no-plough loss",noPloughLoss,textWidth," L/Ha");
		myPrint(Crop.." no-weeding loss",noWeedingLoss,textWidth," L/Ha");
		myPrint(Crop.." liming gain",LimeBonus,textWidth," L/Ha");
		myPrint(Crop.." 1x fertilizing gain",FertilizerBonus,textWidth," L/Ha");
		myPrint(Crop.." max yield",10000*bonusYield,textWidth," L/Ha");
		myPrintFloat(Crop.." hectar per 30 sec",rowgrain.hectarePer30Sec,textWidth," Ha");
		myPrint(Crop.." Liters per 30 sec",10000*bonusYield * rowgrain.hectarePer30Sec,textWidth," L/30 sec\n");
	end;


--POTATO

	idx = 9;
    Crop = Fruits[idx];
	randomYieldFactor = 1 + randomness * 2*(math.random()-0.5) * yieldRange[idx];
	bonusYield = bonusScale * randomYieldFactor * cropYield[idx]*100/(cropDensity[idx]*10000); --Real life yield in liters/sqm
	baseYield = gameCorrection * bonusYield;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('POTATO');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = cropDensity[idx]/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].literPerSqm =  gameLevelYieldCorrection * baseYield;
	noPloughLoss = 10000 * baseYield * 0.15;				-- 15% loss
	noWeedingLoss = 10000 * baseYield * 0.2;				-- 20% loss
	LimeBonus = 10000 * baseYield * 0.15;					-- 15% gain
	FertilizerBonus = 10000 * baseYield * 0.25;				-- 25% gain
	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." massPerLiter",cropDensity[idx],textWidth," kg/L");
		myPrintFloat("Yield randomness scaling",randomYieldFactor,textWidth," ");
		myPrintFloat(Crop.." base yield",baseYield,textWidth," L/sqm");
		myPrint(Crop.." base yield",10000*baseYield,textWidth," L/Ha");
		myPrint(Crop.." no-plough loss",noPloughLoss,textWidth," L/Ha");
		myPrint(Crop.." no-weeding loss",noWeedingLoss,textWidth," L/Ha");
		myPrint(Crop.." liming gain",LimeBonus,textWidth," L/Ha");
		myPrint(Crop.." 1x fertilizing gain",FertilizerBonus,textWidth," L/Ha");
		myPrint(Crop.." max yield",10000*bonusYield,textWidth," L/Ha");
		myPrintFloat(Crop.." hectar per 30 sec",potato.hectarePer30Sec,textWidth," Ha");
		myPrint(Crop.." Liters per 30 sec",10000*bonusYield * potato.hectarePer30Sec,textWidth," L/30 sec\n");
	end;


--SUGARBEET

	idx = 10;
    Crop = Fruits[idx];
	randomYieldFactor = 1 + randomness * 2*(math.random()-0.5) * yieldRange[idx];
	bonusYield = bonusScale * randomYieldFactor * cropYield[idx]*100/(cropDensity[idx]*10000); --Real life yield in liters/sqm
	baseYield = gameCorrection * bonusYield;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('SUGARBEET');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = cropDensity[idx]/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].literPerSqm =  gameLevelYieldCorrection * baseYield;
	noPloughLoss = 10000 * baseYield * 0.15;				-- 15% loss
	noWeedingLoss = 10000 * baseYield * 0.2;				-- 20% loss
	LimeBonus = 10000 * baseYield * 0.15;					-- 15% gain
	FertilizerBonus = 10000 * baseYield * 0.25;				-- 25% gain
 	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." massPerLiter",cropDensity[idx],textWidth," kg/L");
		myPrintFloat("Yield randomness scaling",randomYieldFactor,textWidth," ");
		myPrintFloat(Crop.." base yield",baseYield,textWidth," L/sqm");
		myPrint(Crop.." base yield",10000*baseYield,textWidth," L/Ha");
		myPrint(Crop.." no-plough loss",noPloughLoss,textWidth," L/Ha");
		myPrint(Crop.." no-weeding loss",noWeedingLoss,textWidth," L/Ha");
		myPrint(Crop.." liming gain",LimeBonus,textWidth," L/Ha");
		myPrint(Crop.." 1x fertilizing gain",FertilizerBonus,textWidth," L/Ha");
		myPrint(Crop.." max yield",10000*bonusYield,textWidth," L/Ha");
		myPrintFloat(Crop.." hectar per 30 sec",sugarbeet.hectarePer30Sec,textWidth," Ha");
		myPrint(Crop.." Liters per 30 sec",10000*bonusYield * sugarbeet.hectarePer30Sec,textWidth," L/30 sec\n");
	end;


--OILSEEDRADISH

	idx = 11;
    Crop = Fruits[idx];
	randomYieldFactor = 1 + randomness * 2*(math.random()-0.5) * yieldRange[idx];
	bonusYield = bonusScale * randomYieldFactor * cropYield[idx]*100/(cropDensity[idx]*10000); --Real life yield in liters/sqm
	baseYield = gameCorrection * bonusYield;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('OILSEEDRADISH');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = cropDensity[idx]/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].literPerSqm =  gameLevelYieldCorrection * baseYield;
	noPloughLoss = 10000 * baseYield * 0.15;				-- 15% loss
	noWeedingLoss = 10000 * baseYield * 0.2;				-- 20% loss
	LimeBonus = 10000 * baseYield * 0.15;					-- 15% gain
	FertilizerBonus = 10000 * baseYield * 0.25;				-- 25% gain
	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." massPerLiter",cropDensity[idx],textWidth," kg/L");
		myPrintFloat("Yield randomness scaling",randomYieldFactor,textWidth," ");
		myPrintFloat(Crop.." base yield",baseYield,textWidth," L/sqm");
		myPrint(Crop.." base yield",10000*baseYield,textWidth," L/Ha");
		myPrint(Crop.." no-plough loss",noPloughLoss,textWidth," L/Ha");
		myPrint(Crop.." no-weeding loss",noWeedingLoss,textWidth," L/Ha");
		myPrint(Crop.." liming gain",LimeBonus,textWidth," L/Ha");
		myPrint(Crop.." 1x fertilizing gain",FertilizerBonus,textWidth," L/Ha");
		myPrint(Crop.." max yield",10000*bonusYield,textWidth," L/Ha\n");
	end;


--POPLAR

	idx = 12;
    Crop = Fruits[idx];
	randomYieldFactor = 1 + randomness * 2*(math.random()-0.5) * yieldRange[idx];
	bonusYield = bonusScale * randomYieldFactor * cropYield[idx]*100/(cropDensity[idx]*10000); --Real life yield in liters/sqm
	baseYield = gameCorrection * bonusYield;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('POPLAR');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = cropDensity[idx]/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].literPerSqm =  gameLevelYieldCorrection * baseYield;
	noPloughLoss = 10000 * baseYield * 0.15;				-- 15% loss
	noWeedingLoss = 10000 * baseYield * 0.2;				-- 20% loss
	LimeBonus = 10000 * baseYield * 0.15;					-- 15% gain
	FertilizerBonus = 10000 * baseYield * 0.25;				-- 25% gain
	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." massPerLiter",cropDensity[idx],textWidth," kg/L");
		myPrintFloat("Yield randomness scaling",randomYieldFactor,textWidth," ");
		myPrintFloat(Crop.." base yield",baseYield,textWidth," L/sqm");
		myPrint(Crop.." base yield",10000*baseYield,textWidth," L/Ha");
		myPrint(Crop.." no-plough loss",noPloughLoss,textWidth," L/Ha");
		myPrint(Crop.." no-weeding loss",noWeedingLoss,textWidth," L/Ha");
		myPrint(Crop.." liming gain",LimeBonus,textWidth," L/Ha");
		myPrint(Crop.." 1x fertilizing gain",FertilizerBonus,textWidth," L/Ha");
		myPrint(Crop.." max yield",10000*bonusYield,textWidth," L/Ha\n");
	end;

-- GRASS :: 13

-- RealNumbersGreeenCropYield.lua

-- DRYGRASS :: 14

-- RealNumbersGreeenCropYield.lua

--SUGARCANE

	idx = 15;
    Crop = Fruits[idx];
	randomYieldFactor = 1 + randomness * 2*(math.random()-0.5) * yieldRange[idx];
	bonusYield = bonusScale * randomYieldFactor * cropYield[idx]*100/(cropDensity[idx]*10000); --Real life yield in liters/sqm
	baseYield = gameCorrection * bonusYield;
    iFillType = g_fillTypeManager:getFillTypeIndexByName('SUGARCANE');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = cropDensity[idx]/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].literPerSqm =  gameLevelYieldCorrection * baseYield;
	noPloughLoss = 10000 * baseYield * 0.15;				-- 15% loss
	noWeedingLoss = 10000 * baseYield * 0.2;				-- 20% loss
	LimeBonus = 10000 * baseYield * 0.15;					-- 15% gain
	FertilizerBonus = 10000 * baseYield * 0.25;				-- 25% gain
 	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." massPerLiter",cropDensity[idx],textWidth," kg/L");
		myPrintFloat("Yield randomness scaling",randomYieldFactor,textWidth," ");
		myPrintFloat(Crop.." base yield",baseYield,textWidth," L/sqm");
		myPrint(Crop.." base yield",10000*baseYield,textWidth," L/Ha");
		myPrint(Crop.." no-plough loss",noPloughLoss,textWidth," L/Ha");
		myPrint(Crop.." no-weeding loss",noWeedingLoss,textWidth," L/Ha");
		myPrint(Crop.." liming gain",LimeBonus,textWidth," L/Ha");
		myPrint(Crop.." 1x fertilizing gain",FertilizerBonus,textWidth," L/Ha");
		myPrint(Crop.." max yield",10000*bonusYield,textWidth," L/Ha\n");
	end;

	if isDebug then print("\nRealNumbersCropYield Debug Report End \n") end;
end;


function RealNumbersCropYield:keyEvent(unicode, sym, modifier, isDown)
	local textWidth = 40;
	local isDebug = RN.isDebug;

	local savegameDirectory = g_currentMission.missionInfo.savegameDirectory;	-- works only after savegame is present

	if savegameDirectory ~= nil then

		if Input.isKeyPressed(Input.KEY_lalt) and Input.isKeyPressed(Input.KEY_r) and Input.isKeyPressed(Input.KEY_y) then

			local hour = g_currentMission.environment.currentHour;
			local minute = g_currentMission.environment.currentMinute;
			if isDebug then
				print("Left Alt-ry pressed\n");
				print("Map                   : "..g_currentMission.missionInfo.map.title.." "..g_currentMission.missionInfo.displayName);
				print("Money                 : "..g_currentMission.missionInfo.money);
				print("Economic difficulty   : "..g_currentMission.missionInfo.economicDifficulty );
				print("Current game day      : "..g_currentMission.environment.currentDay);
				print("Time of day           : "..hour..":"..minute);
				print(" ");
			end;
			if g_fruitTypeManager.fruitTypes ~= nil then
				if isDebug then print("\nCrop yield \n");end;
				for k,fruittype in pairs(g_fruitTypeManager.fruitTypes) do
					local fruitname = fruittype.name;
					local literPerHa = 10000 * fruittype.literPerSqm/RN.fieldWorkbaseYieldAdjustment ;
					myPrint(fruitname.." max yield",literPerHa,textWidth," L/Ha");
				end;
				if isDebug then print(" ");end;
			end;
		end;
	else
		print("No save game yet. Save the game, quit and reload");
	end;
end;
function RealNumbersCropYield:update(dt)
end;
function RealNumbersCropYield:draw()
end;
function RealNumbersCropYield:deleteMap()
end;
function RealNumbersCropYield:mouseEvent(posX, posY, isDown, isUp, button)
end;

addModEventListener(RealNumbersCropYield);
