--
-- Mod: FS19_RealLifeNumbers by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- Script: RealNumbersGreenCropYield by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- e-mail: kaj-aage.henneberg@outlook.dk
-- All rights reserved.
-- Version 1.0.0.0
-- 30.11.2018 - 20.03.2019

-- Names for FillTypes

-- WHEAT BARLEY OAT CANOLA SUNFLOWER SOYBEAN MAIZE POTATO SUGARBEET OILSEEDRADISH
-- MANURE SEEDS FORAGE FORAGE_MIXING CHAFF WOODCHIPS SILAGE
-- STRAW GRASS_WINDROW DRYGRASS_WINDROW SUGARCANE FERTILIZER PIGFOOD LIME
-- WOOL TREESAPLINGS EGG
-- MILK FUEL WATER DIESEL DEF
-- LIQUIDFERTILIZER HERBICIDE
-- CHAFF GRASS_WINDROW
-- LIQUIDMANURE DIGESTATE
-- COW_TYPE_BRAHMAN_GREY COW_TYPE_BRAHMAN_LIGHT_BROWN COW_TYPE_BRAHMAN_WHITE
-- COW_TYPE_BRAHMAN_BROWN COW_TYPE_BROWN COW_TYPE_BROWN_WHITE COW_TYPE_BLACK
-- COW_TYPE_BLACK_WHITE
-- SHEEP_TYPE_WHITE SHEEP_TYPE_BROWN SHEEP_TYPE_BLACK_WHITE SHEEP_TYPE_BLACK
-- PIG_TYPE_RED PIG_TYPE_WHITE PIG_TYPE_BLACK_WHITE PIG_TYPE_BLACK
-- CHICKEN_TYPE_BLACK CHICKEN_TYPE_WHITE CHICKEN_TYPE_BROWN CHICKEN_TYPE_ROOSTER

-- FruitType indices

	-- WHEAT :: 1, BARLEY :: 2, OAT :: 3, COTTON :: 4, CANOLA :: 5, SUNFLOWER :: 6,
	-- SOYBEAN :: 7, MAIZE :: 8, POTATO :: 9, SUGARBEET :: 10, OILSEEDRADISH :: 11,
	-- POPLAR :: 12, GRASS :: 13, DRYGRASS :: 14, SUGARCANE :: 15, WEED :: 16


--[[
Crop mass density

	Crop mass density (kg/Liter) varies as a function of moisture content. Different sources will therefore
	provide slightly different numbers.

	Be aware that Farmingsimulator requires density numbers in (metric) Tonne/Liter. Hence the numbers listed
	above is divided by 1000 when assigning the value to massPerLiter.

	Crop mass density is considered independent of geographical location.

Grass Yield

	Grass yield is defined based on kilogram dry matter growth per day per hectare.
	Growing days is by default set to 90 real life days.
	Total fresh matter yield per ha is obtained by dividing the dry matter yield
	by the dry matter fraction of fresh grass (set to 0.33, i.e. moisture = 66%).
	Fresh matter mass is converted to volume by dividing by the mass density of a
	fresh grass bale.

Dry grass yield

	Dry grass (hay) mass per hectare is obtained by dividing the fresh grass dry matter
	yield by the dry matter fraction of hay (set to 0.85, i.e. moisture = 15%).
	Dry grass mass is converted to volume by dividing by the mass density of a
	hay bale.

	A 5% loss is assumed for tedding.


Windrow yield

	A 5% loss is assumed for windrowing both wet and dry grass

	Look in the log file for printed output from this script.
]]--

RealNumbersGreenCropYield = {};


function RealNumbersGreenCropYield:loadMap(name)
    local textWidth = 40; 							-- text width in log file
	local iFillType = 0;
	local iFruitType = 0;
	local Crop = "Empty";
	local idx = 0;
	local baseYield = 0;
	local baseWindrow = 0;
	local bonusScale = 1.0;								-- extra yield for perfect crop management 
	local massScale = 1000;								-- FS uses Tonne/Liter for mass density
	local gameCorrection = RN.fieldWorkbaseYieldAdjustment;	-- scale factor to obtain base yield from bonus yield
	local randomYieldFactor = 1;						-- default value used if reading seed number fails
	local randomness = 1;								-- Set to 1 to turn randomness on, 0 to turn off
	local gameLevelYieldCorrection = RN.gameLevel.cropYieldCorrection;	
--	local grassDryMatterPerDayPerHa 		= 45; 		-- kg DM/day/hay
--	local numberOfGrassGrowingDays 			= 90;		-- Days
	
	local grassBaleVolume 					=  RN.roundBaleVolume[2]; 
	local grassBaleMass 					=  RN.roundBaleMass[2];
	local GrassToHayYieldRatio				=  2.57; 	-- US average
	local dryGrassDryMatterFraction 		=  0.85; 	-- Fraction of dry matter in hay (15% moisture)
	local freshGrassDryMatterFraction 		=  dryGrassDryMatterFraction/GrassToHayYieldRatio; 
	local tedderLoss 						=  0.05;	-- Material loss from tedding
	local windrowLoss 						=  0.05;	-- Material loss from windrowing
	local hayBaleVolume 					=  RN.roundBaleVolume[3];
	local hayBaleMass 						=  RN.roundBaleMass[3];	
	     
	local grassYieldVariation				=  0.153;
	local annualGrassCuttings				=  2;		-- number of grass cuttings per year
	local annualGrassMassPerHa				=  annualGrassCuttings * RN.cropYield[13]*100;   
	
	print("FS19_RealLifeNumbers: RealNumbersGreenCropYield active")

	local isDebug = RN.isDebug;  						-- Change to False when no printing to log file is desired
	local isDebug2 = false;
	

	
-- parameter calculations

	randomYieldFactor = 1 + randomness * 2 *(math.random()- 0.5) * grassYieldVariation;
	local grassBaleDensity = grassBaleMass / grassBaleVolume ;			-- kg/L
	local hayBaleDensity = hayBaleMass / hayBaleVolume;					-- kg/L
--	local grassDryMatterMassPerHa = grassDryMatterPerDayPerHa * numberOfGrassGrowingDays;
--	local grassFreshMatterMassPerHa = grassDryMatterMassPerHa/freshGrassDryMatterFraction;
	local grassFreshMatterMassPerHa = bonusScale * randomYieldFactor * annualGrassMassPerHa / annualGrassCuttings;				
	local freshGrassVolumePerHa =  grassFreshMatterMassPerHa / grassBaleDensity;
	local freshGrassWindrowMassPerHa = (1-windrowLoss) * grassFreshMatterMassPerHa
	local freshGrassWindrowVolumePerHa =  freshGrassWindrowMassPerHa / grassBaleDensity;
	local numberOfFreshGrassBalesPerHa = freshGrassWindrowVolumePerHa/grassBaleVolume;
--	local dryGrassMassPerHa = grassDryMatterMassPerHa/dryGrassDryMatterFraction;
	local dryGrassMassPerHa = (1-tedderLoss) * freshGrassDryMatterFraction * grassFreshMatterMassPerHa/dryGrassDryMatterFraction;
	local dryGrassVolumePerHa = dryGrassMassPerHa / hayBaleDensity;
	local dryGrassWindrowMassPerHa = (1-windrowLoss) * dryGrassMassPerHa
	local dryGrassWindrowVolumePerHa =  dryGrassWindrowMassPerHa / hayBaleDensity;
	local numberOfHayBalesPerHa = dryGrassWindrowVolumePerHa/hayBaleVolume;
	
	RN.grassMassYield = grassFreshMatterMassPerHa/1000;	--Tonne/ha
	RN.freshGrassVolumePerHa = freshGrassVolumePerHa;
	RN.dryGrassWindrowVolumePerHa = dryGrassWindrowVolumePerHa;
	RN.annualGrassCuttings = annualGrassCuttings;

	if isDebug then print("\nRealNumbersGreenCropYield Debug Report Start \n") end;


--GRASS

    Crop = "GRASS";
	baseYield = gameCorrection * freshGrassVolumePerHa/10000;
	baseWindrow = gameCorrection * freshGrassWindrowVolumePerHa/10000;
	iFillType = g_fillTypeManager:getFillTypeIndexByName('GRASS');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = grassBaleDensity/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].literPerSqm =  gameLevelYieldCorrection * baseYield;
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowLiterPerSqm = gameLevelYieldCorrection * baseWindrow;
	if isDebug then
		print("\n---- Per cutting ----\n");
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." massPerLiter",grassBaleDensity,textWidth," kg/L");
		myPrintFloat(Crop.." TonnesPerHa",grassFreshMatterMassPerHa/1000,textWidth," Tonnes/ha");
		myPrint(Crop.." literPerHa",freshGrassVolumePerHa,textWidth," L/ha\n");
	end;


--GRASS_WINDROW

    Crop = "GRASS_WINDROW";
	baseYield = gameCorrection * freshGrassVolumePerHa/10000;
	baseWindrow = gameCorrection * freshGrassWindrowVolumePerHa/10000;
	iFillType = g_fillTypeManager:getFillTypeIndexByName('GRASS_WINDROW');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = grassBaleDensity/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].literPerSqm =  gameLevelYieldCorrection * baseYield;
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowLiterPerSqm = gameLevelYieldCorrection * baseWindrow;
	if isDebug then
		print("\n---- Per cutting ----\n");
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." massPerLiter",grassBaleDensity,textWidth," kg/L");
		myPrintFloat(Crop.." TonnesPerHa",freshGrassWindrowMassPerHa/1000,textWidth," Tonne/ha");
		myPrint(Crop.." windrowLiterPerHa",freshGrassWindrowVolumePerHa,textWidth," L/ha");
		myPrint(Crop.." Bale volume",grassBaleVolume,textWidth," L");
		myPrint(Crop.." Bale mass",grassBaleMass,textWidth," kg");
		myPrintFloat(Crop.." BalesPerHa",numberOfFreshGrassBalesPerHa,textWidth," Bales/ha\n");
	end;


--DRYGRASS


    Crop = "DRYGRASS";
	baseYield = gameCorrection * dryGrassVolumePerHa/10000;
	baseWindrow = gameCorrection * dryGrassWindrowVolumePerHa/10000;
	iFillType = g_fillTypeManager:getFillTypeIndexByName('DRYGRASS');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = hayBaleDensity/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].literPerSqm =  gameLevelYieldCorrection * baseYield;
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowLiterPerSqm = gameLevelYieldCorrection * baseWindrow;
	if isDebug then
		print("\n---- Per cutting ----\n");
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." massPerLiter",hayBaleDensity,textWidth," kg/L");
		myPrintFloat(Crop.." Fresh grass moisture",1-freshGrassDryMatterFraction,textWidth," ");
		myPrintFloat(Crop.." Dry grass moisture",1-dryGrassDryMatterFraction,textWidth," ");
		myPrintFloat(Crop.." Drying mass reduction factor",freshGrassDryMatterFraction/dryGrassDryMatterFraction,textWidth," ");
		myPrintFloat(Crop.." TonnesPerHa",dryGrassMassPerHa/1000,textWidth," Tonne/ha");
		myPrint(Crop.." literPerHa",dryGrassVolumePerHa,textWidth," L/ha\n");
	end;

--DRYGRASS_WINDROW

    Crop = "DRYGRASS_WINDROW";
	baseYield = gameCorrection * dryGrassVolumePerHa/10000;
	baseWindrow = gameCorrection * dryGrassWindrowVolumePerHa/10000;
	iFillType = g_fillTypeManager:getFillTypeIndexByName('DRYGRASS_WINDROW');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = hayBaleDensity/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].literPerSqm =  gameLevelYieldCorrection * baseYield;
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].windrowLiterPerSqm = gameLevelYieldCorrection * baseWindrow;
	if isDebug then
		print("\n---- Per cutting ----\n");
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." massPerLiter",hayBaleDensity,textWidth," kg/L");
		myPrintFloat(Crop.." TonnesPerHa",dryGrassWindrowMassPerHa/1000,textWidth," Tonnes/ha");
		myPrint(Crop.." windrowLiterPerHa",dryGrassWindrowVolumePerHa,textWidth," L/ha");
		myPrint(Crop.." Bale volume",hayBaleVolume,textWidth," L");
		myPrint(Crop.." Bale mass",hayBaleMass,textWidth," kg");
		myPrintFloat(Crop.." BalesPerHa",numberOfHayBalesPerHa,textWidth," Bales/ha\n");
	end;

	if isDebug then print("\nRealNumbersGreenCropYield Debug Report End \n") end;
end;


function RealNumbersGreenCropYield:keyEvent(unicode, sym, modifier, isDown)
end;
function RealNumbersGreenCropYield:update(dt)
end;
function RealNumbersGreenCropYield:draw()
end;
function RealNumbersGreenCropYield:deleteMap()
end;
function RealNumbersGreenCropYield:mouseEvent(posX, posY, isDown, isUp, button)
end;

addModEventListener(RealNumbersGreenCropYield);
