--
-- Mod: FS19_RealLifeNumbers by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- Script: RealNumbersSeedUsage by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- e-mail: kaj-aage.henneberg@outlook.dk
-- All rights reserved.
-- Version 1.0.0.0
-- 30.11.2018 - 20.03.2019

-- Names for FillTypes

-- WHEAT BARLEY OAT CANOLA SUNFLOWER SOYBEAN MAIZE POTATO SUGARBEET OILSEEDRADISH
-- MANURE SEEDS FORAGE FORAGE_MIXING CHAFF WOODCHIPS SILAGE
-- STRAW GRASS_WINDROW DRYGRASS_WINDROW SUGARCANE FERTILIZER PIGFOOD LIME
-- WOOL TREESAPLINGS EGG
-- MILK FUEL WATER DIESEL DEF
-- LIQUIDFERTILIZER HERBICIDE
-- CHAFF GRASS_WINDROW
-- LIQUIDMANURE DIGESTATE
-- COW_TYPE_BRAHMAN_GREY COW_TYPE_BRAHMAN_LIGHT_BROWN COW_TYPE_BRAHMAN_WHITE
-- COW_TYPE_BRAHMAN_BROWN COW_TYPE_BROWN COW_TYPE_BROWN_WHITE COW_TYPE_BLACK
-- COW_TYPE_BLACK_WHITE
-- SHEEP_TYPE_WHITE SHEEP_TYPE_BROWN SHEEP_TYPE_BLACK_WHITE SHEEP_TYPE_BLACK
-- PIG_TYPE_RED PIG_TYPE_WHITE PIG_TYPE_BLACK_WHITE PIG_TYPE_BLACK
-- CHICKEN_TYPE_BLACK CHICKEN_TYPE_WHITE CHICKEN_TYPE_BROWN CHICKEN_TYPE_ROOSTER

-- FruitType indices

	-- WHEAT :: 1, BARLEY :: 2, OAT :: 3, COTTON :: 4, CANOLA :: 5, SUNFLOWER :: 6,
	-- SOYBEAN :: 7, MAIZE :: 8, POTATO :: 9, SUGARBEET :: 10, OILSEEDRADISH :: 11,
	-- POPLAR :: 12, GRASS :: 13, DRYGRASS :: 14, SUGARCANE :: 15, WEED :: 16


--[[
Seed mass density

	Seed mass density (kg/Liter) varies as a function of moisture content. Different sources will therefore
	provide slightly different numbers. The majority of the numbers listed above are taken from
	The agricultural service of the province of Alberta, Canada.

	Be aware that Farmingsimulator requires density numbers in (metric) Ton/Liter. Hence the numbers listed
	above is divided by 1000 when assigning the value to massPerLiter.

	Seed mass density is considered independent of geographical location.


Seed Usage

	Seed usage is in some places given as kg seed per hectare. In FarmingSimulator the
	same information must be given in liters of seed per square meter (seedUsagePerSqm).
	The higher the number, the more seed you need to fill into the seeder/planter for a
	given sized field.

	Seed usage is calculated as:

                               TSM (gram) x plantsPerSqm (1/m^2)
 	 seedUsagePerHa (kg/ha) = ----------------------------------
                                 Germination percentage

	where

		TSM : 						Thousand Seed Mass (the weight in gram of 1000 seed grains
		plantsPerSqm :				Number of plants per square meter
		Germination percentage : 	The number of germinated plants out of 100 seeds

	Example:
	                          30 g      300 pl      100      1 kg    10000 m^2
	seedUsagePerHa (kg/ha) = ------- x --------- x ------ x ------ x --------- = 100 kg/ha
	                         1000 pl     m^2         90     1000 g      1 ha


--]]


RealNumbersSeedUsage = {};


function RealNumbersSeedUsage:loadMap(name)
    local textWidth = 40; 	-- text width in log file
    local iFillType = 0;
	local iFruitType = 0;
	local Crop = "Empty";
	local idx = 0;
	local currency = "€";	-- or  "$" or "£"
	local seedRate = 0;
	local seedMassRatePerHa = 0;
	local seedMassRatePerSqm = 0;
	local seedCostPerHa = 0;
	local massScale = 1000;
	local calibrationTime = 30;
	local isDebug = RN.isDebug;  	-- Change to False when not debugging
	local seedCostSum = 0;			-- summing variable to compute mass weighed mean seed price
	local seedMassSum = 0;			-- summing variable to compute mass weighed mean seed price
	local Fruits = RN.Fruits;
	local seedDensity = RN.seedDensity;
	local TSM = RN.TSM;
	local plantsPerSqm = RN.plantsPerSqm;
	local Germination = RN.Germination;
	local seedPricePer100kg = RN.seedPricePer100kg;

	local gameMoneyUnit = g_i18n.moneyUnit;
	if gameMoneyUnit == 1 then 
        currency = "€";
	elseif gameMoneyUnit == 2 then 
        currency = "$";
	elseif gameMoneyUnit == 3 then 
        currency = "£";
	end;	
	
	print("FS19_RealLifeNumbers: RealNumbersSeedUsage active")

	if isDebug then print("\nRealNumbersSeedUsage Debug Report Start \n") end;



	grainseeder = {};
	rowplanter = {};
	potatoplanter = {};
	
	grainseeder.workWidth = RN.grainseederWorkWidth;										-- Work width in meters of grain seeder
	grainseeder.workingSpeed = RN.grainseederWorkingSpeed;									-- Working speedlimit of seeder in kph
	grainseeder.seedCapacity = RN.grainseederSeedCapacity;									-- Seed capacity in liters
	grainseeder.AreaPerSec = grainseeder.workWidth * grainseeder.workingSpeed/3.6;			-- Area covered per second
	grainseeder.workCapacity = 3600 * grainseeder.AreaPerSec/10000;							-- ha per hour

	rowplanter.workWidth = RN.rowplanterWorkWidth;											-- Work width in meters of grain seeder
	rowplanter.workingSpeed = RN.rowplanterWorkingSpeed;									-- Working speedlimit of seeder in kph
	rowplanter.seedCapacity = RN.rowplanterSeedCapacity;									-- Seed capacity in liters
	rowplanter.AreaPerSec = rowplanter.workWidth * rowplanter.workingSpeed/3.6;				-- Area covered per second
	rowplanter.workCapacity = 3600 * rowplanter.AreaPerSec/10000;							-- ha per hour

	potatoplanter.workWidth = RN.potatoplanterWorkWidth;									-- Work width in meters of grain seeder
	potatoplanter.workingSpeed = RN.potatoplanterWorkingSpeed;								-- Working speedlimit of seeder in kph
	potatoplanter.seedCapacity = RN.potatoplanterSeedCapacity;								-- Seed capacity in liters
	potatoplanter.AreaPerSec = potatoplanter.workWidth * potatoplanter.workingSpeed/3.6;	-- Area covered per second
	potatoplanter.workCapacity = 3600 * potatoplanter.AreaPerSec/10000;						-- ha per hour
	
-- WHEAT

    idx = 1;
    Crop = Fruits[idx];
	grainseeder.wheatCalibrationFactor = RN.seederCalibration[idx];
	seedMassRatePerHa = grainseeder.wheatCalibrationFactor * (TSM[idx]*plantsPerSqm[idx])/Germination[idx];  -- kg/ha
	seedMassRatePersqm = seedMassRatePerHa/10000;
	seedCostPerHa = seedPricePer100kg[idx] * seedMassRatePerHa/100;
	seedRate = seedMassRatePersqm / seedDensity[idx];
    iFillType = g_fillTypeManager:getFillTypeIndexByName('WHEAT');							-- get fillType by crop name
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);			-- get fruitType by fillType
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].seedUsagePerSqm = seedRate;
	grainseeder.litersPerSecond = grainseeder.AreaPerSec * seedRate;
	grainseeder.calibrationDose = calibrationTime * grainseeder.litersPerSecond;			-- Target dose spread in calibration time
	grainseeder.calibrationArea = calibrationTime * grainseeder.AreaPerSec;					-- Area covered in calibration time
	seedCostSum = seedCostSum + seedMassRatePerHa * seedCostPerHa;
	seedMassSum = seedMassSum + seedMassRatePerHa;

	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." seedMassPerLiter",seedDensity[idx],textWidth," kg/L");
		myPrint(Crop.." seedMassPerHa",seedMassRatePerHa,textWidth," kg/Ha");
		myPrint(Crop.." seedCostPerHa",seedCostPerHa,textWidth,currency.."/Ha");
		myPrint(Crop.." seedUsagePerHa",10000*seedRate,textWidth," L/Ha");
		myPrintFloat(Crop.." seedUsagePerSqm",seedRate,textWidth," L/sqm");
		myPrintFloat("Area per sec",grainseeder.AreaPerSec,textWidth," sqm/s");
		myPrintFloat("Seeding rate",grainseeder.litersPerSecond,textWidth," L/s");
		myPrintFloat("Calibration dosage",grainseeder.calibrationDose,textWidth," L");
		myPrintFloat("Calibration area",grainseeder.calibrationArea,textWidth," sqm");
		myPrintFloat("Calibration factor",grainseeder.wheatCalibrationFactor,textWidth," ");
		print(" ");
	end;

--BARLEY

    idx = 2;
    Crop = Fruits[idx];
	grainseeder.barleyCalibrationFactor = RN.seederCalibration[idx];
	seedMassRatePerHa = grainseeder.barleyCalibrationFactor * (TSM[idx]*plantsPerSqm[idx])/Germination[idx];  -- kg/ha
	seedMassRatePersqm = seedMassRatePerHa/10000;
	seedCostPerHa = seedPricePer100kg[idx] * seedMassRatePerHa/100;
	seedRate = seedMassRatePersqm / seedDensity[idx];
    iFillType = g_fillTypeManager:getFillTypeIndexByName('BARLEY');
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].seedUsagePerSqm = seedRate;
	grainseeder.litersPerSecond = grainseeder.AreaPerSec * seedRate;
	grainseeder.calibrationDose = calibrationTime * grainseeder.litersPerSecond;			-- Target dose spread in calibration time
	grainseeder.calibrationArea = calibrationTime * grainseeder.AreaPerSec;					-- Area covered in calibration time
	seedCostSum = seedCostSum + seedMassRatePerHa * seedCostPerHa;
	seedMassSum = seedMassSum + seedMassRatePerHa;
	
	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." seedMassPerLiter",seedDensity[idx],textWidth," kg/L");
		myPrint(Crop.." seedMassPerHa",seedMassRatePerHa,textWidth," kg/Ha");
		myPrint(Crop.." seedCostPerHa",seedCostPerHa,textWidth,currency.."/Ha");
		myPrint(Crop.." seedUsagePerHa",10000*seedRate,textWidth," L/Ha");
		myPrintFloat(Crop.." seedUsagePerSqm",seedRate,textWidth," L/sqm");
		myPrintFloat("Area per sec",grainseeder.AreaPerSec,textWidth," sqm/s");
		myPrintFloat("Seeding rate",grainseeder.litersPerSecond,textWidth," L/s");
		myPrintFloat("Calibration dosage",grainseeder.calibrationDose,textWidth," L");
		myPrintFloat("Calibration area",grainseeder.calibrationArea,textWidth," sqm");
		myPrintFloat("Calibration factor",grainseeder.barleyCalibrationFactor,textWidth," ");
		print(" ");
	end;


--OAT

	idx = 3;
    Crop = Fruits[idx];
	grainseeder.oatCalibrationFactor = RN.seederCalibration[idx];
	seedMassRatePerHa = grainseeder.oatCalibrationFactor * (TSM[idx]*plantsPerSqm[idx])/Germination[idx];  -- kg/ha
	seedMassRatePersqm = seedMassRatePerHa/10000;
	seedCostPerHa = seedPricePer100kg[idx] * seedMassRatePerHa/100;
	seedRate = seedMassRatePersqm / seedDensity[idx];
    iFillType = g_fillTypeManager:getFillTypeIndexByName('OAT');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = seedDensity[idx]/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].seedUsagePerSqm = seedRate;
	grainseeder.litersPerSecond = grainseeder.AreaPerSec * seedRate;
	grainseeder.calibrationDose = calibrationTime * grainseeder.litersPerSecond;			-- Target dose spread in calibration time
	grainseeder.calibrationArea = calibrationTime * grainseeder.AreaPerSec;					-- Area covered in calibration time
	seedCostSum = seedCostSum + seedMassRatePerHa * seedCostPerHa;
	seedMassSum = seedMassSum + seedMassRatePerHa;

	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." seedMassPerLiter",seedDensity[idx],textWidth," kg/L");
		myPrint(Crop.." seedMassPerHa",seedMassRatePerHa,textWidth," kg/Ha");
		myPrint(Crop.." seedCostPerHa",seedCostPerHa,textWidth,currency.."/Ha");
		myPrint(Crop.." seedUsagePerHa",10000*seedRate,textWidth," L/Ha");
		myPrintFloat(Crop.." seedUsagePerSqm",seedRate,textWidth," L/sqm");
		myPrintFloat("Area per sec",grainseeder.AreaPerSec,textWidth," sqm/s");
		myPrintFloat("Seeding rate",grainseeder.litersPerSecond,textWidth," L/s");
		myPrintFloat("Calibration dosage",grainseeder.calibrationDose,textWidth," L");
		myPrintFloat("Calibration area",grainseeder.calibrationArea,textWidth," sqm");
		myPrintFloat("Calibration factor",grainseeder.oatCalibrationFactor,textWidth," ");
		print(" ");
	end;
	
-- COTTON

-- No known numbers

--CANOLA

	idx = 5;
    Crop = Fruits[idx];
	grainseeder.canolaCalibrationFactor = RN.seederCalibration[idx];
	seedMassRatePerHa = grainseeder.canolaCalibrationFactor * (TSM[idx]*plantsPerSqm[idx])/Germination[idx];  -- kg/ha
	seedMassRatePersqm = seedMassRatePerHa/10000;
	seedCostPerHa = seedPricePer100kg[idx] * seedMassRatePerHa/100;
	seedRate = seedMassRatePersqm / seedDensity[idx];
    iFillType = g_fillTypeManager:getFillTypeIndexByName('CANOLA');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = seedDensity[idx]/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].seedUsagePerSqm = seedRate;
	grainseeder.litersPerSecond = grainseeder.AreaPerSec * seedRate;
	grainseeder.calibrationDose = calibrationTime * grainseeder.litersPerSecond;			-- Target dose spread in calibration time
	grainseeder.calibrationArea = calibrationTime * grainseeder.AreaPerSec;					-- Area covered in calibration time
	seedCostSum = seedCostSum + seedMassRatePerHa * seedCostPerHa;
	seedMassSum = seedMassSum + seedMassRatePerHa;
	
	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." seedMassPerLiter",seedDensity[idx],textWidth," kg/L");
		myPrint(Crop.." seedMassPerHa",seedMassRatePerHa,textWidth," kg/Ha");
		myPrint(Crop.." seedCostPerHa",seedCostPerHa,textWidth,currency.."/Ha");
		myPrint(Crop.." seedUsagePerHa",10000*seedRate,textWidth," L/Ha");
		myPrintFloat(Crop.." seedUsagePerSqm",seedRate,textWidth," L/sqm");
		myPrintFloat("Area per sec",grainseeder.AreaPerSec,textWidth," sqm/s");
		myPrintFloat("Seeding rate",grainseeder.litersPerSecond,textWidth," L/s");
		myPrintFloat("Calibration dosage",grainseeder.calibrationDose,textWidth," L");
		myPrintFloat("Calibration area",grainseeder.calibrationArea,textWidth," sqm");
		myPrintFloat("Calibration factor",grainseeder.canolaCalibrationFactor,textWidth," ");
		print(" ");
	end;

--SUNFLOWER

	idx = 6;
    Crop = Fruits[idx];
	rowplanter.sunflowerCalibrationFactor = RN.seederCalibration[idx];
	seedMassRatePerHa = rowplanter.sunflowerCalibrationFactor * (TSM[idx]*plantsPerSqm[idx])/Germination[idx];  -- kg/ha
	seedMassRatePersqm = seedMassRatePerHa/10000;
	seedCostPerHa = seedPricePer100kg[idx] * seedMassRatePerHa/100;
	seedRate = seedMassRatePersqm / seedDensity[idx];
    iFillType = g_fillTypeManager:getFillTypeIndexByName('SUNFLOWER');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = seedDensity[idx]/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].seedUsagePerSqm = seedRate;
	rowplanter.litersPerSecond = rowplanter.AreaPerSec * seedRate;
	rowplanter.calibrationDose = calibrationTime * rowplanter.litersPerSecond;				-- Target dose spread in calibration time
	rowplanter.calibrationArea = calibrationTime * rowplanter.AreaPerSec;					-- Area covered in calibration time
	seedCostSum = seedCostSum + seedMassRatePerHa * seedCostPerHa;
	seedMassSum = seedMassSum + seedMassRatePerHa;

	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." seedMassPerLiter",seedDensity[idx],textWidth," kg/L");
		myPrint(Crop.." seedMassPerHa",seedMassRatePerHa,textWidth," kg/Ha");
		myPrint(Crop.." seedCostPerHa",seedCostPerHa,textWidth,currency.."/Ha");
		myPrint(Crop.." seedUsagePerHa",10000*seedRate,textWidth," L/Ha");
		myPrintFloat(Crop.." seedUsagePerSqm",seedRate,textWidth," L/sqm");
		myPrintFloat("Area per sec",rowplanter.AreaPerSec,textWidth," sqm/s");
		myPrintFloat("Seeding rate",rowplanter.litersPerSecond,textWidth," L/s");
		myPrintFloat("Calibration dosage",rowplanter.calibrationDose,textWidth," L");
		myPrintFloat("Calibration area",rowplanter.calibrationArea,textWidth," sqm");
		myPrintFloat("Calibration factor",rowplanter.sunflowerCalibrationFactor,textWidth," ");
		print(" ");
	end;

--SOYBEAN

	idx = 7;
    Crop = Fruits[idx];
	rowplanter.soybeanCalibrationFactor = RN.seederCalibration[idx];
	seedMassRatePerHa = rowplanter.soybeanCalibrationFactor * (TSM[idx]*plantsPerSqm[idx])/Germination[idx];  -- kg/ha
	seedMassRatePersqm = seedMassRatePerHa/10000;
	seedCostPerHa = seedPricePer100kg[idx] * seedMassRatePerHa/100;
	seedRate = seedMassRatePersqm / seedDensity[idx];
    iFillType = g_fillTypeManager:getFillTypeIndexByName('SOYBEAN');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = seedDensity[idx]/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].seedUsagePerSqm = seedRate;
	rowplanter.litersPerSecond = rowplanter.AreaPerSec * seedRate;
	rowplanter.calibrationDose = calibrationTime * rowplanter.litersPerSecond;				-- Target dose spread in calibration time
	rowplanter.calibrationArea = calibrationTime * rowplanter.AreaPerSec;					-- Area covered in calibration time
	seedCostSum = seedCostSum + seedMassRatePerHa * seedCostPerHa;
	seedMassSum = seedMassSum + seedMassRatePerHa;

	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." seedMassPerLiter",seedDensity[idx],textWidth," kg/L");
		myPrint(Crop.." seedMassPerHa",seedMassRatePerHa,textWidth," kg/Ha");
		myPrint(Crop.." seedCostPerHa",seedCostPerHa,textWidth,currency.."/Ha");
		myPrint(Crop.." seedUsagePerHa",10000*seedRate,textWidth," L/Ha");
		myPrintFloat(Crop.." seedUsagePerSqm",seedRate,textWidth," L/sqm");
		myPrintFloat("Area per sec",rowplanter.AreaPerSec,textWidth," sqm/s");
		myPrintFloat("Seeding rate",rowplanter.litersPerSecond,textWidth," L/s");
		myPrintFloat("Calibration dosage",rowplanter.calibrationDose,textWidth," L");
		myPrintFloat("Calibration area",rowplanter.calibrationArea,textWidth," sqm");
		myPrintFloat("Calibration factor",rowplanter.soybeanCalibrationFactor,textWidth," ");
		print(" ");
	end;

--MAIZE

	idx = 8;
    Crop = Fruits[idx];
	rowplanter.maizeCalibrationFactor = RN.seederCalibration[idx];
	seedMassRatePerHa = rowplanter.maizeCalibrationFactor * (TSM[idx]*plantsPerSqm[idx])/Germination[idx];  -- kg/ha
	seedMassRatePersqm = seedMassRatePerHa/10000;
	seedCostPerHa = seedPricePer100kg[idx] * seedMassRatePerHa/100;
	seedRate = seedMassRatePersqm / seedDensity[idx];
    iFillType = g_fillTypeManager:getFillTypeIndexByName('MAIZE');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = seedDensity[idx]/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].seedUsagePerSqm = seedRate;
	rowplanter.litersPerSecond = rowplanter.AreaPerSec * seedRate;
	rowplanter.calibrationDose = calibrationTime * rowplanter.litersPerSecond;				-- Target dose spread in calibration time
	rowplanter.calibrationArea = calibrationTime * rowplanter.AreaPerSec;					-- Area covered in calibration time
	seedCostSum = seedCostSum + seedMassRatePerHa * seedCostPerHa;
	seedMassSum = seedMassSum + seedMassRatePerHa;

	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." seedMassPerLiter",seedDensity[idx],textWidth," kg/L");
		myPrint(Crop.." seedMassPerHa",seedMassRatePerHa,textWidth," kg/Ha");
		myPrint(Crop.." seedCostPerHa",seedCostPerHa,textWidth,currency.."/Ha");
		myPrint(Crop.." seedUsagePerHa",10000*seedRate,textWidth," L/Ha");
		myPrintFloat(Crop.." seedUsagePerSqm",seedRate,textWidth," L/sqm");
		myPrintFloat("Area per sec",rowplanter.AreaPerSec,textWidth," sqm/s");
		myPrintFloat("Seeding rate",rowplanter.litersPerSecond,textWidth," L/s");
		myPrintFloat("Calibration dosage",rowplanter.calibrationDose,textWidth," L");
		myPrintFloat("Calibration area",rowplanter.calibrationArea,textWidth," sqm");
		myPrintFloat("Calibration factor",rowplanter.maizeCalibrationFactor,textWidth," ");
		print(" ");
	end;

--POTATO

	idx = 9;
    Crop = Fruits[idx];
	potatoplanter.potatoCalibrationFactor = RN.seederCalibration[idx];
	seedMassRatePerHa = potatoplanter.potatoCalibrationFactor * (TSM[idx]*plantsPerSqm[idx])/Germination[idx];  -- kg/ha
	seedMassRatePersqm = seedMassRatePerHa/10000;
	seedCostPerHa = seedPricePer100kg[idx] * seedMassRatePerHa/100;
	seedRate = seedMassRatePersqm / seedDensity[idx];
    iFillType = g_fillTypeManager:getFillTypeIndexByName('POTATO');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = seedDensity[idx]/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].seedUsagePerSqm = seedRate;
	potatoplanter.litersPerSecond = potatoplanter.AreaPerSec * seedRate;
	potatoplanter.calibrationDose = calibrationTime * potatoplanter.litersPerSecond;		-- Target dose spread in calibration time
	potatoplanter.calibrationArea = calibrationTime * potatoplanter.AreaPerSec;				-- Area covered in calibration time

	-- do not include potatoes in the calculation of seed price
	-- their heavy mass will dominate completely

	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." seedMassPerLiter",seedDensity[idx],textWidth," kg/L");
		myPrint(Crop.." seedMassPerHa",seedMassRatePerHa,textWidth," kg/Ha");
		myPrint(Crop.." seedCostPerHa",seedCostPerHa,textWidth,currency.."/Ha");
		myPrint(Crop.." seedUsagePerHa",10000*seedRate,textWidth," L/Ha");
		myPrintFloat(Crop.." seedUsagePerSqm",seedRate,textWidth," L/sqm");
		myPrintFloat("Area per sec",potatoplanter.AreaPerSec,textWidth," sqm/s");
		myPrintFloat("Seeding rate",potatoplanter.litersPerSecond,textWidth," L/s");
		myPrintFloat("Calibration dosage",potatoplanter.calibrationDose,textWidth," L");
		myPrintFloat("Calibration area",potatoplanter.calibrationArea,textWidth," sqm");
		myPrintFloat("Calibration factor",potatoplanter.potatoCalibrationFactor,textWidth," ");
		print(" ");
	end;

--SUGARBEET

	idx = 10;
    Crop = Fruits[idx];
	rowplanter.sugarbeetCalibrationFactor = RN.seederCalibration[idx];
	seedMassRatePerHa = rowplanter.sugarbeetCalibrationFactor * (TSM[idx]*plantsPerSqm[idx])/Germination[idx];  -- kg/ha
	seedMassRatePersqm = seedMassRatePerHa/10000;
	seedCostPerHa = seedPricePer100kg[idx] * seedMassRatePerHa/100;
	seedRate = seedMassRatePersqm / seedDensity[idx];
    iFillType = g_fillTypeManager:getFillTypeIndexByName('SUGARBEET');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = seedDensity[idx]/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].seedUsagePerSqm = seedRate;
	rowplanter.litersPerSecond = rowplanter.AreaPerSec * seedRate;
	rowplanter.calibrationDose = calibrationTime * rowplanter.litersPerSecond;				-- Target dose spread in calibration time
	rowplanter.calibrationArea = calibrationTime * rowplanter.AreaPerSec;					-- Area covered in calibration time
	seedCostSum = seedCostSum + seedMassRatePerHa * seedCostPerHa;
	seedMassSum = seedMassSum + seedMassRatePerHa;

	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." seedMassPerLiter",seedDensity[idx],textWidth," kg/L");
		myPrint(Crop.." seedMassPerHa",seedMassRatePerHa,textWidth," kg/Ha");
		myPrint(Crop.." seedCostPerHa",seedCostPerHa,textWidth,currency.."/Ha");
		myPrint(Crop.." seedUsagePerHa",10000*seedRate,textWidth," L/Ha");
		myPrintFloat(Crop.." seedUsagePerSqm",seedRate,textWidth," L/sqm");
		myPrintFloat("Area per sec",rowplanter.AreaPerSec,textWidth," sqm/s");
		myPrintFloat("Seeding rate",rowplanter.litersPerSecond,textWidth," L/s");
		myPrintFloat("Calibration dosage",rowplanter.calibrationDose,textWidth," L");
		myPrintFloat("Calibration area",rowplanter.calibrationArea,textWidth," sqm");
		myPrintFloat("Calibration factor",rowplanter.sugarbeetCalibrationFactor,textWidth," ");
		print(" ");
	end;

--OILSEEDRADISH

	idx = 11;
    Crop = Fruits[idx];
	grainseeder.oilseedradishCalibrationFactor = RN.seederCalibration[idx];
	seedMassRatePerHa = grainseeder.oilseedradishCalibrationFactor * (TSM[idx]*plantsPerSqm[idx])/Germination[idx];  -- kg/ha
	seedMassRatePersqm = seedMassRatePerHa/10000;
	seedCostPerHa = seedPricePer100kg[idx] * seedMassRatePerHa/100;
	seedRate = seedMassRatePersqm / seedDensity[idx];
    iFillType = g_fillTypeManager:getFillTypeIndexByName('OILSEEDRADISH');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = seedDensity[idx]/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].seedUsagePerSqm = seedRate;
	grainseeder.litersPerSecond = grainseeder.AreaPerSec * seedRate;
	grainseeder.calibrationDose = calibrationTime * grainseeder.litersPerSecond;			-- Target dose spread in calibration time
	grainseeder.calibrationArea = calibrationTime * grainseeder.AreaPerSec;					-- Area covered in calibration time
	seedCostSum = seedCostSum + seedMassRatePerHa * seedCostPerHa;
	seedMassSum = seedMassSum + seedMassRatePerHa;

	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." seedMassPerLiter",seedDensity[idx],textWidth," kg/L");
		myPrint(Crop.." seedMassPerHa",seedMassRatePerHa,textWidth," kg/Ha");
		myPrint(Crop.." seedCostPerHa",seedCostPerHa,textWidth,currency.."/Ha");
		myPrint(Crop.." seedUsagePerHa",10000*seedRate,textWidth," L/Ha");
		myPrintFloat(Crop.." seedUsagePerSqm",seedRate,textWidth," L/sqm");
		myPrintFloat("Area per sec",grainseeder.AreaPerSec,textWidth," sqm/s");
		myPrintFloat("Seeding rate",grainseeder.litersPerSecond,textWidth," L/s");
		myPrintFloat("Calibration dosage",grainseeder.calibrationDose,textWidth," L");
		myPrintFloat("Calibration area",grainseeder.calibrationArea,textWidth," sqm");
		myPrintFloat("Calibration factor",grainseeder.oilseedradishCalibrationFactor,textWidth," ");
		print(" ");
	end;

	
--GRASS

	idx = 13;
    Crop = Fruits[idx];
	grainseeder.grassCalibrationFactor = RN.seederCalibration[idx];
	seedMassRatePerHa = grainseeder.grassCalibrationFactor * (TSM[idx]*plantsPerSqm[idx])/Germination[idx];  -- kg/ha
	seedMassRatePersqm = seedMassRatePerHa/10000;
	seedCostPerHa = seedPricePer100kg[idx] * seedMassRatePerHa/100;
	seedRate = seedMassRatePersqm / seedDensity[idx];
    iFillType = g_fillTypeManager:getFillTypeIndexByName('GRASS');
	g_currentMission.fillTypeManager.fillTypes[iFillType].massPerLiter = seedDensity[idx]/massScale;
	iFruitType = g_fruitTypeManager:getFruitTypeIndexByFillTypeIndex(iFillType);
	g_currentMission.fruitTypeManager.fruitTypes[iFruitType].seedUsagePerSqm = seedRate;
	grainseeder.litersPerSecond = grainseeder.AreaPerSec * seedRate;
	grainseeder.calibrationDose = calibrationTime * grainseeder.litersPerSecond;			-- Target dose spread in calibration time
	grainseeder.calibrationArea = calibrationTime * grainseeder.AreaPerSec;					-- Area covered in calibration time
	seedCostSum = seedCostSum + seedMassRatePerHa * seedCostPerHa;
	seedMassSum = seedMassSum + seedMassRatePerHa;

	if isDebug then
		myPrint(Crop.." fillTypeIndex",iFillType,textWidth," ");
		myPrint(Crop.." fruitTypeIndex",iFruitType,textWidth," ");
		myPrintFloat(Crop.." seedMassPerLiter",seedDensity[idx],textWidth," kg/L");
		myPrint(Crop.." seedMassPerHa",seedMassRatePerHa,textWidth," kg/Ha");
		myPrint(Crop.." seedCostPerHa",seedCostPerHa,textWidth,currency.."/Ha");
		myPrint(Crop.." seedUsagePerHa",10000*seedRate,textWidth," L/Ha");
		myPrintFloat(Crop.." seedUsagePerSqm",seedRate,textWidth," L/sqm");
		myPrintFloat("Area per sec",grainseeder.AreaPerSec,textWidth," sqm/s");
		myPrintFloat("Seeding rate",grainseeder.litersPerSecond,textWidth," L/s");
		myPrintFloat("Calibration dosage",grainseeder.calibrationDose,textWidth," L");
		myPrintFloat("Calibration area",grainseeder.calibrationArea,textWidth," sqm");
		myPrintFloat("Calibration factor",grainseeder.grassCalibrationFactor,textWidth," ");
		print(" ");
	end;
	
	if isDebug then myPrintFloat("Default seed price",RN.commodityPricePer100kg[3],textWidth,currency.."/100kg"); end;
	RN.commodityPricePer100kg[3] = seedCostSum/seedMassSum;		-- setting mean seed price based on mass averaged seed cost per ha
	if isDebug then myPrintFloat("Adjudsted seed price",RN.commodityPricePer100kg[3],textWidth,currency.."/100kg"); end;

	if isDebug then print("\nRealNumbersSeedUsage Debug Report End \n") end; 
end;


function RealNumbersSeedUsage:keyEvent(unicode, sym, modifier, isDown)
end;
function RealNumbersSeedUsage:update(dt)
end;
function RealNumbersSeedUsage:draw()
end;
function RealNumbersSeedUsage:deleteMap()
end;
function RealNumbersSeedUsage:mouseEvent(posX, posY, isDown, isUp, button)
end;

addModEventListener(RealNumbersSeedUsage);
