--
-- Mod: FS19_RealLifeNumbers by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- Script: RealNumbersSpraying by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- e-mail: kaj-aage.henneberg@outlook.dk
-- All rights reserved.
-- Version 1.0.0.0
-- 30.11.2018 - 20.03.2019

-- Names for FillTypes

-- WHEAT BARLEY OAT CANOLA SUNFLOWER SOYBEAN MAIZE POTATO SUGARBEET OILSEEDRADISH
-- MANURE SEEDS FORAGE FORAGE_MIXING CHAFF WOODCHIPS SILAGE
-- STRAW GRASS_WINDROW DRYGRASS_WINDROW SUGARCANE FERTILIZER PIGFOOD LIME
-- WOOL TREESAPLINGS EGG
-- MILK FUEL WATER DIESEL DEF
-- LIQUIDFERTILIZER HERBICIDE
-- CHAFF GRASS_WINDROW
-- LIQUIDMANURE DIGESTATE
-- COW_TYPE_BRAHMAN_GREY COW_TYPE_BRAHMAN_LIGHT_BROWN COW_TYPE_BRAHMAN_WHITE COW_TYPE_BRAHMAN_BROWN
-- COW_TYPE_BROWN COW_TYPE_BROWN_WHITE COW_TYPE_BLACK COW_TYPE_BLACK_WHITE
-- SHEEP_TYPE_WHITE SHEEP_TYPE_BROWN SHEEP_TYPE_BLACK_WHITE SHEEP_TYPE_BLACK
-- PIG_TYPE_RED PIG_TYPE_WHITE PIG_TYPE_BLACK_WHITE PIG_TYPE_BLACK
-- CHICKEN_TYPE_BLACK CHICKEN_TYPE_WHITE CHICKEN_TYPE_BROWN CHICKEN_TYPE_ROOSTER

--[[

Crop nutrient need

	The strategy of this script is to add as much nutrient as is taken up by the crops and removed from the field at harvest.
	A crop's uptake of nutrients (nitrogen (N), phospate (P), and potassium (K)) depends on the yield of each crop. Data for nutrient
	uptake is tken from a USDA database, where the uptake per mass is dependent on moisture content in the crop. Linear regression
	was used to obtain a moisture scaling model, so that the mod user can change the moisture content of the crop, if so desired.
	Many countries, but not all, require N, P and K to be listed in specific forms, such as N, P2O5, and K2O. This convention is used
	in this script.

	The nutrient uptake per hectar of a crop is calculated using the crop yield parameters. A table is written to the log file
	displaying the NPK removal per hectar for each crop. The mod does not define fertlizer spray rate based on crop type. There is
	only one spray rate in the mod. Hence, assuming some degree of crop rotation over many seasons, the mod defines the NPK need per
	hectar as the average of the need of the individual crops. The weakness of this approach is that the set NPK need per hectar
	may be influenced by crops that you never grow.

	The removal of nutrients is calculated separately for grain and straw.

NPK content in organic fertlizers

	If one were to use organic fertilizers (slurry, manure, and digestate) to cover the total need for NPK replacement, one would
	need many tank loads per hectar. For this reason a parameter (nutrient.organicFertilizerFraction) is used to split the applied
	fertilizer between organic and artificial. The default split is 20% organic and 80% artificial. This plit ratio can be changed
	by the mod user. This approach implies that the first application of NPK is by an organic source and the second by an artificial
	fertilizer.

	The NPK content of organic fertilizers varies. In FS we have five types available: pig slurry, cow slurry, pig manure, cow manure,
	and digestate (from silage fermentation). For slurry and manure, the mod user can set a mixing ratio betwwen pig and cow wastes:

	nutrient.pigSlurryFraction = 0.5, This implies that 50% of the slurry spread on the fields is from pigs and 50% from cows.
	nutrient.pigManureFraction = 0.5, This implies that 50% of the manure spread on the fields is from pigs and 50% from cows.

	If your slurry and manure productions are dominated by one type of animal, you can change the two fractions. Information about
	your annual slurry and manure production is provided in log print-outs from another script: RealNumbersAnimalCare.

	Far from all nutrients in a fertilizer is aborbed by the crop. The uptake fraction varies both between the nutients (N, P, K) and between
	the types and sources of organic fertilizers (slurry/manure, pig/cow). This is included in the script by parameters as shown here:

	nutrient.pigSlurryNCropAvailability = 0.55;
	nutrient.pigSlurryPCropAvailability = 0.50;
	nutrient.pigSlurryKCropAvailability = 0.90;
	nutrient.cowManureNCropAvailability = 0.10;
	nutrient.cowManurePCropAvailability = 0.60;
	nutrient.cowManureKCropAvailability = 0.90;

	It is seen that the N utilization from cow manure is particularly low, due to gasseous nitrogen release to the air.

	Using the mixing ratios and the crop-availablility paramters, the script defines the NPK content in a representative slurry/manure.

Nutrients delivered

	The ratio of N, P, and K, needed are different from the ratios in the fertilizer sources, hence the script calculates the
	organic fertilizer dose to meet the need of phosphate. Most contries have fertilizer acts regulating how much N, P, and K can
	be applied. Phosphate is typically the nutrient with the most strict limitation. The script then calculates how much N, P, and K
	is delivered per hectar, when the need to phosphate is met.

	The amount of artificial fertilizer to apply is calculated as the gap between the needed potassium (K) and the average applied from
	slurry, manure and digestate.

	The amount of liquid fertilizer needed is less than tht for solid fertilizer, as the nutrients come in direct contact with the plants.
	This is included in the script using the parameter (liqfert.liquidReductionFactor).

	Liquid fertilizers must be diluted with water for the sprayer to work effectively. The spray rate parameter must take this into account.
	The script provides a parameter (liqfert.dilutionFactor) to set the dilution factor. It is set so that 1 Liter of liquid fertilizer is
	diluted to a 20 Liter solution.

Lime application

	Lime is spread to raise the pH of the soil. How much to spread depends on (1) how much the pH needs to be rised, (2) on the soil type,
	and (3) on the lime source.

	The script allows the user to define these parameters:

	soil.Type = 1;				 		-- 1: Sands and loamy sands: 2: Sandy loams and silt loam; 3: Clay loams and clay
	soil.currentpH = 6;
	soil.targetpH = 6.7;
	lime.NV = 50;  				 		-- (NV = Neutralizing Value) concentration of calcium in lime material, 0 - 100

Herbicide application

	Herbicides are sprayed to kill weeds. In this script, the default amount is 2 kg/ha, but this can be changed by the mod user.
	Herbicides are usually diluted with water and sprayed with a liquid fertilizer sprayer. In the script the default dilution is set
	by the parameter:

	herbicide.dilutionFactor = 200;		-- 1 liter of herbicide to 200 liter of water

Setting of in-game spray rates

	By spray rate is meant how many liters per second is broadcast by the spreader/sprayer. To calculate this, we need to apply some
	simple math:

			LitersPerSec = LitersPerSqm * AreaPerSec

			LitersPerSqm = LitersPerHa / 10000

	To calculate AreaPerSec we need some informaton about the spreader/sprayer:

			AreaPerSec = SprayWidth * WorkingSpeed/3.6

	If the working speed is 18 kph and the spraywidth is 12 m, then the sprayer covers 60 sqm/s.
	Besides the working speed and the spraywidth, the script also needs the load capacity of the spreader/sprayer in Liter. Using this value,
	the script calculates how many hectars can be covered in one load. Ths is important information when you need to decide on the size of
	your spreader/sprayer.

	In Farming Simulator the spray rate parameter is to be understood as "Liters per square meter" even though the parameter is called
	litersPerSecond. Hence the spray rate is obtained directly from the desired application:

	litersPerSqm = LitersPerHectar/10000
	correctedSprayRate = litersPerSqm/3.6

	There seems to be an error in how FS19 calculates sprayType Usage. If one looks into the function: Sprayer:getSprayerUsage(fillType, dt)
	the last line says:

	scale * litersPerSecond * self.speedLimit * workWidth * dt * 0.001

	I assume the speedLimit is obtained from the implement XML, where speedLimit is given in kph. In the lua code above the speedLimit
	should have been divided by 3.6 to get speed in m/s. As the code neglects this, the calculated usage is 3.6 times greater than it should
	have been. To compensate for this, this script divides the parameter LitersPerSecond by 3.6.

Sprayer calibration

	In real life, farmers need to calibrate sprayers and spreaders so that the desired dosage is applied.  Typically such a calibration is
	done by driving the spreader/sprayer at a certain velocity and then measure the time. If you know the time, driving speed and spray width,
	you can calculate the area covered. If you also make samples of the amount sprayed, you can calculate the dosage delivered. If this is below
	or above the target value, the sprayer/spreader needs to be adjusted.

	When using this script, you can make a calibration much like what real life farmers do. The script sets the test run to 30 seconds
	and writes the target spray volume for the sprayer/spreader to the log file. You will then make a run down the test field at maximum working
	speed and make notes of the amount before and after the test run. The difference is the amount spread in 30 seconds. This amount should then
	be compared to the target test dosage written to the log file. From this the calibrationFactor for the sprayer/spreader can be defined:

	calibrationFactor = measured dosage / target dosage

	The calibration method is most accurate if you can drive down the field with maximum work speed. This means that you should avoid using a
	hilly field for the calibration and use the biggest tractor you own. Using a count down timer on a smart phone is perfect for measuring the
	30 sec run time. 30 sec is sufficiently long to minimize errors from a standing start and sufficiently short that you can use many medium
	sized fields. If the field is too short, you can drive around in circles. The spraying will be oddly distributed, but the calibration is
	valid.

	Before doing the calibration, you must enter work width, work speed and capacity of the sprayer/spreader into the script and reinstall
	the script. You need to enter these data for all your sprayers and spreaders and do a calibration for each.

Calibration verifiction

	When each sprayer/spreader has been calibrated, the calibration needs to be verified by doing another test run. If the delivered dosage
	is as close to the targt dosage as you would like it to be, the calibration is completed.

Final comments

	Look in the log file for printed output from this script.

]]--

RealNumbersSpraying = {};

function myPrintSprayInfo(crop, textWidth, Moisture, N, P, K)
	local text = crop..string.rep(" ",textWidth-string.len(crop))
	print(string.format("%s: Moisture pct = %7.1f, N = %7.1f kg/ha, P2O5 = %7.1f kg/ha, K2O = %7.1f kg/ha", text, Moisture, N, P, K));
end;
function myPrintSprayInfo2(crop, textWidth, N, P, K)
	local text = crop..string.rep(" ",textWidth-string.len(crop))
	print(string.format("%s: N = %7.1f kg/ha, P2O5 = %7.1f kg/ha, K2O = %7.1f kg/ha", text, N, P, K));
end;
function myPrintNutrientContent(source, textWidth, N, P, K)
	local text = source..string.rep(" ",textWidth-string.len(source))
	print(string.format("%s: N = %9.1f kg/Tonne, P2O5 = %9.1f kg/Tonne, K2O = %9.1f kg/Tonne", text, N, P, K));
end;
function myPrintNutrientContent2(source, textWidth, N, P, K)
	local text = source..string.rep(" ",textWidth-string.len(source))
	print(string.format("%s: N = %7.0f   L/Ha    , P2O5 = %7.0f   L/Ha    , K2O = %7.0f   L/Ha    ", text, N, P, K));
end;


function RealNumbersSpraying:loadMap()
	local currency = " ";
	local isDebug = RN.isDebug;
	local isDebug2 = false;
	local idx = 0;
	local modName = "FS19_RealLifeNumbers_DE";
	local RealLifeNumbersDirectory = g_modNameToDirectory[modName];
	local moddir = string.gsub(RealLifeNumbersDirectory, "/"..modName, "");
    local textWidth = 15;
    local textWidth2 = 35;
	local textWidth3 = 40;
	local nutrient = {};												--parameters for nutrient content in crops and fertilizers
	local slurry = {};													--parameters for slurry sprayers
	local manure = {};													--parameters for manure spreader
	local digestate = {};												--parameters for digestate sprayers
	local fertilizer = {};												--parameters for fertilizer spreader
	local liqfert = {};													--parameters for liquid fertilizer sprayer
	local lime = {};													--parameters for lime spreader
	local herbicide = {};												--parameters for herbicide sprayer
	local soil = {};
	local Fruits = RN.Fruits;
	local Density = RN.cropDensity;
	local Yield = RN.cropYield;

	local sprayNames =  RN.sprayNames;
	local sprayTypeDensity = RN.sprayTypeDensity;
	local sprayRates =      {0.0060,        0.0081,        0.4000,     0.4000,        0.4000,    0.0900,    0.0081};		--original values, not used here
	local calibrationTime = 30;											--Time duration of calibration run.

	nutrient.slurryDensity = RN.sprayTypeDensity[4];										-- kg/Liter
	nutrient.manureDensity = RN.sprayTypeDensity[3];										-- kg/Liter
	nutrient.digestateDensity = RN.sprayTypeDensity[5]; 									-- kg/Liter
	nutrient.pigSlurryFraction = RN.pigSlurryFraction;									-- fraction of slurry spread on fields originating from pigs
	nutrient.cowSlurryFraction = 1 - nutrient.pigSlurryFraction;		-- fraction of slurry spread on fields originating from cows
	nutrient.pigManureFraction = RN.pigManureFraction;									-- fraction of manure spread on fields originating from pigs
	nutrient.cowManureFraction = 1 - nutrient.pigManureFraction;		-- fraction of manure spread on fields originating from cows
	nutrient.organicFertilizerFraction = RN.organicFertilizerFraction;							-- fraction of fertilizer spread on fields originating from organic sources
	nutrient.artificialNFraction = 14/100;								-- NPK ratio 14:7:14
	nutrient.artificialPFraction = 7/100;								-- NPK ratio 14:7:14
	nutrient.artificialKFraction = 14/100;								-- NPK ratio 14:7:14
	liqfert.dilutionFactor = RN.liqfertDilutionFactor;										-- 1 Liter of liquid fertilizer is diluted to 20 liter of solution
	liqfert.liquidReductionFactor = RN.liqfertLiquidReductionFactor;	-- liquid fertilizer is needed in smaller dosis due to better uptake
	herbicide.dilutionFactor = RN.herbicideDilutionFactor;				-- 1 liter of herbicide to 200 liter of water
	RN.sprayVolume = {};												-- Volume of sprayTypes per hectare
	
	local gameMoneyUnit = g_i18n.moneyUnit;
	if gameMoneyUnit == 1 then
        currency = "€";
	elseif gameMoneyUnit == 2 then
        currency = "$";
	elseif gameMoneyUnit == 3 then
        currency = "£";
	end


	print("FS19_RealLifeNumbers: RealNumbersSpraying active")

	if isDebug then
		print("\nRealNumbersSpraying Debug Report Start \n");
		print("\nAmount of N, P and K fertilzer removed from the field at harvest. \nDepends on crop yield and crop moisture concent.\n")
	end;

-- Summing up crop need to calcule mean value

	local NSum = 0;
	local PSum = 0;
	local KSum = 0;
	local counter = 0;

-- wheat

	idx = 1;
	nutrient.wheatNslope = -0.0002088;									-- moisture dependency parameters
	nutrient.wheatPslope = -0.0000768;
	nutrient.wheatKslope = -0.0000383;
	nutrient.wheatNintercept = 0.0208834;
	nutrient.wheatPintercept = 0.0076869;
	nutrient.wheatKintercept = 0.0038311;
	nutrient.wheatMoisture = 11.45;
	nutrient.wheatNtakeoutPerLiter = (nutrient.wheatNintercept + nutrient.wheatNslope * nutrient.wheatMoisture);
	nutrient.wheatPtakeoutPerLiter = (nutrient.wheatPintercept + nutrient.wheatPslope * nutrient.wheatMoisture);
	nutrient.wheatKtakeoutPerLiter = (nutrient.wheatKintercept + nutrient.wheatKslope * nutrient.wheatMoisture);
	nutrient.wheatYieldVolume = 100 * Yield[idx]/Density[idx];
	nutrient.wheatNtakeout = nutrient.wheatNtakeoutPerLiter * nutrient.wheatYieldVolume;
	nutrient.wheatPtakeout = nutrient.wheatPtakeoutPerLiter * nutrient.wheatYieldVolume;
	nutrient.wheatKtakeout = nutrient.wheatKtakeoutPerLiter * nutrient.wheatYieldVolume;
	if isDebug then myPrintSprayInfo(Fruits[idx], textWidth,nutrient.wheatMoisture, nutrient.wheatNtakeout, nutrient.wheatPtakeout, nutrient.wheatKtakeout); end;
	NSum = NSum + nutrient.wheatNtakeout;
	PSum = PSum + nutrient.wheatPtakeout;
	KSum = KSum + nutrient.wheatKtakeout;
	counter = counter + 1;


-- barley

	idx = 2;
	nutrient.barleyNslope = -0.000130;									-- moisture dependency parameters
	nutrient.barleyPslope = -0.000059;
	nutrient.barleyKslope = -0.000040;
	nutrient.barleyNintercept = 0.013033;
	nutrient.barleyPintercept = 0.005933;
	nutrient.barleyKintercept = 0.004015;
	nutrient.barleyMoisture = 11.72;
	nutrient.barleyNtakeoutPerLiter = (nutrient.barleyNintercept + nutrient.barleyNslope * nutrient.barleyMoisture);
	nutrient.barleyPtakeoutPerLiter = (nutrient.barleyPintercept + nutrient.barleyPslope * nutrient.barleyMoisture);
	nutrient.barleyKtakeoutPerLiter = (nutrient.barleyKintercept + nutrient.barleyKslope * nutrient.barleyMoisture);
	nutrient.barleyYieldVolume = 100 * Yield[idx]/Density[idx];
	nutrient.barleyNtakeout = nutrient.barleyNtakeoutPerLiter * nutrient.barleyYieldVolume;
	nutrient.barleyPtakeout = nutrient.barleyPtakeoutPerLiter * nutrient.barleyYieldVolume;
	nutrient.barleyKtakeout = nutrient.barleyKtakeoutPerLiter * nutrient.barleyYieldVolume;
	if isDebug then myPrintSprayInfo(Fruits[idx], textWidth, nutrient.barleyMoisture, nutrient.barleyNtakeout, nutrient.barleyPtakeout, nutrient.barleyKtakeout); end;
	NSum = NSum + nutrient.barleyNtakeout;
	PSum = PSum + nutrient.barleyPtakeout;
	KSum = KSum + nutrient.barleyKtakeout;
	counter = counter + 1;

-- oat

	idx = 3;
	nutrient.oatNslope = -8.6094E-05;									-- moisture dependency parameters
	nutrient.oatPslope = -3.6046E-05;
	nutrient.oatKslope = -2.3184E-05;
	nutrient.oatNintercept = 0.008621105;
	nutrient.oatPintercept = 0.003602204;
	nutrient.oatKintercept = 0.002313874;
	nutrient.oatMoisture = 10.67;
	nutrient.oatNtakeoutPerLiter = (nutrient.oatNintercept + nutrient.oatNslope * nutrient.oatMoisture);
	nutrient.oatPtakeoutPerLiter = (nutrient.oatPintercept + nutrient.oatPslope * nutrient.oatMoisture);
	nutrient.oatKtakeoutPerLiter = (nutrient.oatKintercept + nutrient.oatKslope * nutrient.oatMoisture);
	nutrient.oatYieldVolume = 100 * Yield[idx]/Density[idx];
	nutrient.oatNtakeout = nutrient.oatNtakeoutPerLiter * nutrient.oatYieldVolume;
	nutrient.oatPtakeout = nutrient.oatPtakeoutPerLiter * nutrient.oatYieldVolume;
	nutrient.oatKtakeout = nutrient.oatKtakeoutPerLiter * nutrient.oatYieldVolume;
	if isDebug then myPrintSprayInfo(Fruits[idx], textWidth, nutrient.oatMoisture, nutrient.oatNtakeout, nutrient.oatPtakeout, nutrient.oatKtakeout); end;
	NSum = NSum + nutrient.oatNtakeout;
	PSum = PSum + nutrient.oatPtakeout;
	KSum = KSum + nutrient.oatKtakeout;
	counter = counter + 1;

-- cotton

	idx = 4;
	nutrient.cottonNslope = -0.0003300;									-- moisture dependency parameters
	nutrient.cottonPslope = -0.0000981;
	nutrient.cottonKslope = -0.0000605;
	nutrient.cottonNintercept = 0.0329533;
	nutrient.cottonPintercept = 0.0094326;
	nutrient.cottonKintercept = 0.005929;
	nutrient.cottonMoisture = 9;
	nutrient.cottonNtakeoutPerKg = (nutrient.cottonNintercept + nutrient.cottonNslope * nutrient.cottonMoisture);
	nutrient.cottonPtakeoutPerKg = (nutrient.cottonPintercept + nutrient.cottonPslope * nutrient.cottonMoisture);
	nutrient.cottonKtakeoutPerKg = (nutrient.cottonKintercept + nutrient.cottonKslope * nutrient.cottonMoisture);
	nutrient.cottonYieldMass = 100 * Yield[idx];
	nutrient.cottonNtakeout = nutrient.cottonNtakeoutPerKg * nutrient.cottonYieldMass;
	nutrient.cottonPtakeout = nutrient.cottonPtakeoutPerKg * nutrient.cottonYieldMass;
	nutrient.cottonKtakeout = nutrient.cottonKtakeoutPerKg * nutrient.cottonYieldMass;
	if isDebug then myPrintSprayInfo(Fruits[idx], textWidth, nutrient.cottonMoisture, nutrient.cottonNtakeout, nutrient.cottonPtakeout, nutrient.cottonKtakeout); end;
	NSum = NSum + nutrient.cottonNtakeout;
	PSum = PSum + nutrient.cottonPtakeout;
	KSum = KSum + nutrient.cottonKtakeout;
	counter = counter + 1;

-- canola

	idx = 5;
	nutrient.canolaNslope = -0.0003889;									-- moisture dependency parameters
	nutrient.canolaPslope = -0.0001420;
	nutrient.canolaKslope = -0.0001186;
	nutrient.canolaNintercept = 0.0388874;
	nutrient.canolaPintercept = 0.0141980;
	nutrient.canolaKintercept = 0.0118580;
	nutrient.canolaMoisture = 8.93;
	nutrient.canolaNtakeoutPerKg = (nutrient.canolaNintercept + nutrient.canolaNslope * nutrient.canolaMoisture);
	nutrient.canolaPtakeoutPerKg = (nutrient.canolaPintercept + nutrient.canolaPslope * nutrient.canolaMoisture);
	nutrient.canolaKtakeoutPerKg = (nutrient.canolaKintercept + nutrient.canolaKslope * nutrient.canolaMoisture);
	nutrient.canolaYieldMass = 100 * Yield[idx];
	nutrient.canolaNtakeout = nutrient.canolaNtakeoutPerKg * nutrient.canolaYieldMass;
	nutrient.canolaPtakeout = nutrient.canolaPtakeoutPerKg * nutrient.canolaYieldMass;
	nutrient.canolaKtakeout = nutrient.canolaKtakeoutPerKg * nutrient.canolaYieldMass;
	if isDebug then myPrintSprayInfo(Fruits[idx], textWidth, nutrient.canolaMoisture, nutrient.canolaNtakeout, nutrient.canolaPtakeout, nutrient.canolaKtakeout); end;
	NSum = NSum + nutrient.canolaNtakeout;
	PSum = PSum + nutrient.canolaPtakeout;
	KSum = KSum + nutrient.canolaKtakeout;
	counter = counter + 1;

-- sunflower

	idx = 6;
	nutrient.sunflowerNslope = -0.0002994;								-- moisture dependency parameters
	nutrient.sunflowerPslope = -0.0001434;
	nutrient.sunflowerKslope = -0.0000869;
	nutrient.sunflowerNintercept = 0.0299324;
	nutrient.sunflowerPintercept = 0.0143439;
	nutrient.sunflowerKintercept = 0.0086971;
	nutrient.sunflowerMoisture = 7.36;
	nutrient.sunflowerNtakeoutPerKg = (nutrient.sunflowerNintercept + nutrient.sunflowerNslope * nutrient.sunflowerMoisture);
	nutrient.sunflowerPtakeoutPerKg = (nutrient.sunflowerPintercept + nutrient.sunflowerPslope * nutrient.sunflowerMoisture);
	nutrient.sunflowerKtakeoutPerKg = (nutrient.sunflowerKintercept + nutrient.sunflowerKslope * nutrient.sunflowerMoisture);
	nutrient.sunflowerYieldMass = 100 * Yield[idx];
	nutrient.sunflowerNtakeout = nutrient.sunflowerNtakeoutPerKg * nutrient.sunflowerYieldMass;
	nutrient.sunflowerPtakeout = nutrient.sunflowerPtakeoutPerKg * nutrient.sunflowerYieldMass;
	nutrient.sunflowerKtakeout = nutrient.sunflowerKtakeoutPerKg * nutrient.sunflowerYieldMass;
	if isDebug then myPrintSprayInfo(Fruits[idx], textWidth, nutrient.sunflowerMoisture, nutrient.sunflowerNtakeout, nutrient.sunflowerPtakeout, nutrient.sunflowerKtakeout); end;
	NSum = NSum + nutrient.sunflowerNtakeout;
	PSum = PSum + nutrient.sunflowerPtakeout;
	KSum = KSum + nutrient.sunflowerKtakeout;
	counter = counter + 1;

-- soybean

	idx = 7;
	nutrient.soybeanNslope = -5.07151E-04;								-- moisture dependency parameters
	nutrient.soybeanPslope = -1.19043E-04;
	nutrient.soybeanKslope = -1.44068E-04;
	nutrient.soybeanNintercept = 0.050722848;
	nutrient.soybeanPintercept = 0.011898619;
	nutrient.soybeanKintercept = 0.014406828;
	nutrient.soybeanMoisture = 10.12;
	nutrient.soybeanNtakeoutPerLiter = (nutrient.soybeanNintercept + nutrient.soybeanNslope * nutrient.soybeanMoisture);
	nutrient.soybeanPtakeoutPerLiter = (nutrient.soybeanPintercept + nutrient.soybeanPslope * nutrient.soybeanMoisture);
	nutrient.soybeanKtakeoutPerLiter = (nutrient.soybeanKintercept + nutrient.soybeanKslope * nutrient.soybeanMoisture);
	nutrient.soybeanYieldVolume = 100 * Yield[idx]/Density[idx];
	nutrient.soybeanNtakeout = nutrient.soybeanNtakeoutPerLiter * nutrient.soybeanYieldVolume;
	nutrient.soybeanPtakeout = nutrient.soybeanPtakeoutPerLiter * nutrient.soybeanYieldVolume;
	nutrient.soybeanKtakeout = nutrient.soybeanKtakeoutPerLiter * nutrient.soybeanYieldVolume;
	if isDebug then myPrintSprayInfo(Fruits[idx], textWidth, nutrient.soybeanMoisture, nutrient.soybeanNtakeout, nutrient.soybeanPtakeout, nutrient.soybeanKtakeout); end;
	NSum = NSum + nutrient.soybeanNtakeout;
	PSum = PSum + nutrient.soybeanPtakeout;
	KSum = KSum + nutrient.soybeanKtakeout;
	counter = counter + 1;

-- maize

	idx = 8;
	nutrient.maizeNslope = -0.0001180;									-- moisture dependency parameters
	nutrient.maizePslope = -0.0000516;
	nutrient.maizeKslope = -0.0000301;
	nutrient.maizeNintercept = 0.011802;
	nutrient.maizePintercept = 0.0051584;
	nutrient.maizeKintercept = 0.0030047;
	nutrient.maizeMoisture = 13.52;
	nutrient.maizeNtakeoutPerLiter = (nutrient.maizeNintercept + nutrient.maizeNslope * nutrient.maizeMoisture);
	nutrient.maizePtakeoutPerLiter = (nutrient.maizePintercept + nutrient.maizePslope * nutrient.maizeMoisture);
	nutrient.maizeKtakeoutPerLiter = (nutrient.maizeKintercept + nutrient.maizeKslope * nutrient.maizeMoisture);
	nutrient.maizeYieldVolume = 100 * Yield[idx]/Density[idx];
	nutrient.maizeNtakeout = nutrient.maizeNtakeoutPerLiter * nutrient.maizeYieldVolume;
	nutrient.maizePtakeout = nutrient.maizePtakeoutPerLiter * nutrient.maizeYieldVolume;
	nutrient.maizeKtakeout = nutrient.maizeKtakeoutPerLiter * nutrient.maizeYieldVolume;
	if isDebug then myPrintSprayInfo(Fruits[idx], textWidth, nutrient.maizeMoisture, nutrient.maizeNtakeout, nutrient.maizePtakeout, nutrient.maizeKtakeout); end;
	NSum = NSum + nutrient.maizeNtakeout;
	PSum = PSum + nutrient.maizePtakeout;
	KSum = KSum + nutrient.maizeKtakeout;
	counter = counter + 1;

-- potato

	idx = 9;
	nutrient.potatoNslope = -0.0001608;									-- moisture dependency parameters
	nutrient.potatoPslope = -0.0000566;
	nutrient.potatoKslope = -0.0002417;
	nutrient.potatoNintercept = 0.0160815;
	nutrient.potatoPintercept = 0.0056565;
	nutrient.potatoKintercept = 0.0241715;
	nutrient.potatoMoisture = 77.2;
	nutrient.potatoNtakeoutPerKg = (nutrient.potatoNintercept + nutrient.potatoNslope * nutrient.potatoMoisture);
	nutrient.potatoPtakeoutPerKg = (nutrient.potatoPintercept + nutrient.potatoPslope * nutrient.potatoMoisture);
	nutrient.potatoKtakeoutPerKg = (nutrient.potatoKintercept + nutrient.potatoKslope * nutrient.potatoMoisture);
	nutrient.potatoYieldMass = 100 * Yield[idx];
	nutrient.potatoNtakeout = nutrient.potatoNtakeoutPerKg * nutrient.potatoYieldMass;
	nutrient.potatoPtakeout = nutrient.potatoPtakeoutPerKg * nutrient.potatoYieldMass;
	nutrient.potatoKtakeout = nutrient.potatoKtakeoutPerKg * nutrient.potatoYieldMass;
	if isDebug then myPrintSprayInfo(Fruits[idx], textWidth, nutrient.potatoMoisture, nutrient.potatoNtakeout, nutrient.potatoPtakeout, nutrient.potatoKtakeout); end;
	NSum = NSum + nutrient.potatoNtakeout;
	PSum = PSum + nutrient.potatoPtakeout;
	KSum = KSum + nutrient.potatoKtakeout;
	counter = counter + 1;

-- sugarbeet

	idx = 10;
	nutrient.sugarbeetNslope = -0.105200;								-- moisture dependency parameters
	nutrient.sugarbeetPslope = -0.049807;
	nutrient.sugarbeetKslope = -0.186744;
	nutrient.sugarbeetNintercept = 10.52;
	nutrient.sugarbeetPintercept = 4.980749;
	nutrient.sugarbeetKintercept = 18.67442;
	nutrient.sugarbeetMoisture = 79.28;
	nutrient.sugarbeetNtakeoutPerTonne = (nutrient.sugarbeetNintercept + nutrient.sugarbeetNslope * nutrient.sugarbeetMoisture);
	nutrient.sugarbeetPtakeoutPerTonne = (nutrient.sugarbeetPintercept + nutrient.sugarbeetPslope * nutrient.sugarbeetMoisture);
	nutrient.sugarbeetKtakeoutPerTonne = (nutrient.sugarbeetKintercept + nutrient.sugarbeetKslope * nutrient.sugarbeetMoisture);
	nutrient.sugarbeetYieldMass = 0.1 * Yield[idx];
	nutrient.sugarbeetNtakeout = nutrient.sugarbeetNtakeoutPerTonne * nutrient.sugarbeetYieldMass;
	nutrient.sugarbeetPtakeout = nutrient.sugarbeetPtakeoutPerTonne * nutrient.sugarbeetYieldMass;
	nutrient.sugarbeetKtakeout = nutrient.sugarbeetKtakeoutPerTonne * nutrient.sugarbeetYieldMass;
	if isDebug then myPrintSprayInfo(Fruits[idx], textWidth, nutrient.sugarbeetMoisture, nutrient.sugarbeetNtakeout, nutrient.sugarbeetPtakeout, nutrient.sugarbeetKtakeout); end;
	NSum = NSum + nutrient.sugarbeetNtakeout;
	PSum = PSum + nutrient.sugarbeetPtakeout;
	KSum = KSum + nutrient.sugarbeetKtakeout;
	counter = counter + 1;

-- grass

	idx = 13;
	nutrient.grassNslope = -0.1863114;									-- moisture dependency parameters
	nutrient.grassPslope = -0.06448;
	nutrient.grassKslope = -0.30839;
	nutrient.grassNintercept = 18.6311747;
	nutrient.grassPintercept = 6.447882;
	nutrient.grassKintercept = 30.83871;
	nutrient.grassMoisture = 85.00;
	nutrient.grassNtakeoutPerTonne = (nutrient.grassNintercept + nutrient.grassNslope * nutrient.grassMoisture);
	nutrient.grassPtakeoutPerTonne = (nutrient.grassPintercept + nutrient.grassPslope * nutrient.grassMoisture);
	nutrient.grassKtakeoutPerTonne = (nutrient.grassKintercept + nutrient.grassKslope * nutrient.grassMoisture);
	nutrient.grassYieldMass = RN.grassMassYield;
	nutrient.grassNtakeout = nutrient.grassNtakeoutPerTonne * nutrient.grassYieldMass;
	nutrient.grassPtakeout = nutrient.grassPtakeoutPerTonne * nutrient.grassYieldMass;
	nutrient.grassKtakeout = nutrient.grassKtakeoutPerTonne * nutrient.grassYieldMass;
	if isDebug then myPrintSprayInfo(Fruits[idx], textWidth, nutrient.grassMoisture, nutrient.grassNtakeout, nutrient.grassPtakeout, nutrient.grassKtakeout); end;
	NSum = NSum + nutrient.grassNtakeout;
	PSum = PSum + nutrient.grassPtakeout;
	KSum = KSum + nutrient.grassKtakeout;
	counter = counter + 1;

-- sugarcane

	idx = 15;
	nutrient.sugarcaneNslope = -0.086600;								-- moisture dependency parameters
	nutrient.sugarcanePslope = -0.053815;
	nutrient.sugarcaneKslope = -0.358160;
	nutrient.sugarcaneNintercept = 8.659998;
	nutrient.sugarcanePintercept = 5.381498;
	nutrient.sugarcaneKintercept = 35.81599;
	nutrient.sugarcaneMoisture = 73.83;
	nutrient.sugarcaneNtakeoutPerTonne = (nutrient.sugarcaneNintercept + nutrient.sugarcaneNslope * nutrient.sugarcaneMoisture);
	nutrient.sugarcanePtakeoutPerTonne = (nutrient.sugarcanePintercept + nutrient.sugarcanePslope * nutrient.sugarcaneMoisture);
	nutrient.sugarcaneKtakeoutPerTonne = (nutrient.sugarcaneKintercept + nutrient.sugarcaneKslope * nutrient.sugarcaneMoisture);
	nutrient.sugarcaneYieldMass = 0.1 * Yield[idx];
	nutrient.sugarcaneNtakeout = nutrient.sugarcaneNtakeoutPerTonne * nutrient.sugarcaneYieldMass;
	nutrient.sugarcanePtakeout = nutrient.sugarcanePtakeoutPerTonne * nutrient.sugarcaneYieldMass;
	nutrient.sugarcaneKtakeout = nutrient.sugarcaneKtakeoutPerTonne * nutrient.sugarcaneYieldMass;
	if isDebug then myPrintSprayInfo(Fruits[idx], textWidth, nutrient.sugarcaneMoisture, nutrient.sugarcaneNtakeout, nutrient.sugarcanePtakeout, nutrient.sugarcaneKtakeout); end;
	NSum = NSum + nutrient.sugarcaneNtakeout;
	PSum = PSum + nutrient.sugarcanePtakeout;
	KSum = KSum + nutrient.sugarcaneKtakeout;
	counter = counter + 1;

-- WHEAT STRAW

	nutrient.wheatStrawNslope = -0.0000980;								-- moisture dependency parameters
	nutrient.wheatStrawPslope = -0.0000224;
	nutrient.wheatStrawKslope = -0.0002179;
	nutrient.wheatStrawNintercept = 0.0098015;
	nutrient.wheatStrawPintercept = 0.002244;
	nutrient.wheatStrawKintercept = 0.0217827;
	nutrient.wheatStrawMoisture = 9.22;
	nutrient.wheatStrawNtakeoutPerKg = (nutrient.wheatStrawNintercept + nutrient.wheatStrawNslope * nutrient.wheatStrawMoisture);
	nutrient.wheatStrawPtakeoutPerKg = (nutrient.wheatStrawPintercept + nutrient.wheatStrawPslope * nutrient.wheatStrawMoisture);
	nutrient.wheatStrawKtakeoutPerKg = (nutrient.wheatStrawKintercept + nutrient.wheatStrawKslope * nutrient.wheatStrawMoisture);
	nutrient.wheatStrawNtakeout = nutrient.wheatStrawNtakeoutPerKg * RN.wheatWindrowYieldkgPerHa;
	nutrient.wheatStrawPtakeout = nutrient.wheatStrawPtakeoutPerKg * RN.wheatWindrowYieldkgPerHa;
	nutrient.wheatStrawKtakeout = nutrient.wheatStrawKtakeoutPerKg * RN.wheatWindrowYieldkgPerHa;
	if isDebug then myPrintSprayInfo("WHEAT STRAW", textWidth, nutrient.wheatStrawMoisture, nutrient.wheatStrawNtakeout, nutrient.wheatStrawPtakeout, nutrient.wheatStrawKtakeout); end;
	NSum = NSum + nutrient.wheatStrawNtakeout;
	PSum = PSum + nutrient.wheatStrawPtakeout;
	KSum = KSum + nutrient.wheatStrawKtakeout;
	counter = counter + 1;

-- BARLEY STRAW

	nutrient.barleyStrawNslope = -0.0001029;							-- moisture dependency parameters
	nutrient.barleyStrawPslope = -0.0000328;
	nutrient.barleyStrawKslope = -0.0003256;
	nutrient.barleyStrawNintercept = 0.0102879;
	nutrient.barleyStrawPintercept = 0.0032819;
	nutrient.barleyStrawKintercept = 0.0325517;
	nutrient.barleyStrawMoisture = 11.25;
	nutrient.barleyStrawNtakeoutPerKg = (nutrient.barleyStrawNintercept + nutrient.barleyStrawNslope * nutrient.barleyStrawMoisture);
	nutrient.barleyStrawPtakeoutPerKg = (nutrient.barleyStrawPintercept + nutrient.barleyStrawPslope * nutrient.barleyStrawMoisture);
	nutrient.barleyStrawKtakeoutPerKg = (nutrient.barleyStrawKintercept + nutrient.barleyStrawKslope * nutrient.barleyStrawMoisture);
	nutrient.barleyStrawNtakeout = nutrient.barleyStrawNtakeoutPerKg * RN.wheatWindrowYieldkgPerHa;
	nutrient.barleyStrawPtakeout = nutrient.barleyStrawPtakeoutPerKg * RN.wheatWindrowYieldkgPerHa;
	nutrient.barleyStrawKtakeout = nutrient.barleyStrawKtakeoutPerKg * RN.wheatWindrowYieldkgPerHa;
	if isDebug then myPrintSprayInfo("BARLEY STRAW", textWidth, nutrient.barleyStrawMoisture, nutrient.barleyStrawNtakeout, nutrient.barleyStrawPtakeout, nutrient.barleyStrawKtakeout); end;
	NSum = NSum + nutrient.barleyStrawNtakeout;
	PSum = PSum + nutrient.barleyStrawPtakeout;
	KSum = KSum + nutrient.barleyStrawKtakeout;
	counter = counter + 1;

-- OAT STRAW

	nutrient.oatStrawNslope = -0.0000914;								-- moisture dependency parameters
	nutrient.oatStrawPslope = -0.0000251;
	nutrient.oatStrawKslope = -0.0003738;
	nutrient.oatStrawNintercept = 0.0091328;
	nutrient.oatStrawPintercept = 0.0025173;
	nutrient.oatStrawKintercept = 0.0373824;
	nutrient.oatStrawMoisture = 9.68;
	nutrient.oatStrawNtakeoutPerKg = (nutrient.oatStrawNintercept + nutrient.oatStrawNslope * nutrient.oatStrawMoisture);
	nutrient.oatStrawPtakeoutPerKg = (nutrient.oatStrawPintercept + nutrient.oatStrawPslope * nutrient.oatStrawMoisture);
	nutrient.oatStrawKtakeoutPerKg = (nutrient.oatStrawKintercept + nutrient.oatStrawKslope * nutrient.oatStrawMoisture);
	nutrient.oatStrawNtakeout = nutrient.oatStrawNtakeoutPerKg * RN.wheatWindrowYieldkgPerHa;
	nutrient.oatStrawPtakeout = nutrient.oatStrawPtakeoutPerKg * RN.wheatWindrowYieldkgPerHa;
	nutrient.oatStrawKtakeout = nutrient.oatStrawKtakeoutPerKg * RN.wheatWindrowYieldkgPerHa;
	if isDebug then myPrintSprayInfo("OAT STRAW", textWidth, nutrient.oatStrawMoisture, nutrient.oatStrawNtakeout, nutrient.oatStrawPtakeout, nutrient.oatStrawKtakeout); end;
	NSum = NSum + nutrient.oatStrawNtakeout;
	PSum = PSum + nutrient.oatStrawPtakeout;
	KSum = KSum + nutrient.oatStrawKtakeout;
	counter = counter + 1;
	local NMean = NSum/counter;
	local PMean = PSum/counter;
	local KMean = KSum/counter;
	if isDebug then print(" ");end;
	if isDebug then myPrintSprayInfo("Mean values", textWidth, 0, NMean, PMean, KMean); end;

	if isDebug then print("\n------------Organic fertilizer nutrient content------------\n"); end;

-- Calculation of nutrient content in organic fertilizers

-- Slurry

--pigs

	if isDebug then print("Fraction of fertilizer from organic sources: "..nutrient.organicFertilizerFraction); end;
    nutrient.pigSlurryNPerTonne = 3.600;	-- kg/Tonne
	nutrient.pigSlurryPPerTonne = 1.500;	-- kg/Tonne
	nutrient.pigSlurryKPerTonne = 2.200;	-- kg/Tonne
	if isDebug then myPrintNutrientContent("pigSlurry", textWidth2, nutrient.pigSlurryNPerTonne, nutrient.pigSlurryPPerTonne, nutrient.pigSlurryKPerTonne); end;
	nutrient.pigSlurryNCropAvailability = 0.55;
	nutrient.pigSlurryPCropAvailability = 0.50;
	nutrient.pigSlurryKCropAvailability = 0.90;
	nutrient.pigSlurryNAvailablePerTonne = nutrient.pigSlurryNCropAvailability * nutrient.pigSlurryNPerTonne;
	nutrient.pigSlurryPAvailablePerTonne = nutrient.pigSlurryPCropAvailability * nutrient.pigSlurryPPerTonne;
	nutrient.pigSlurryKAvailablePerTonne = nutrient.pigSlurryKCropAvailability * nutrient.pigSlurryKPerTonne;
	if isDebug then myPrintNutrientContent("pigSlurry crop available", textWidth2, nutrient.pigSlurryNAvailablePerTonne, nutrient.pigSlurryPAvailablePerTonne, nutrient.pigSlurryKAvailablePerTonne); end;
	nutrient.pigSlurryNTonnePerHa = NMean / nutrient.pigSlurryNAvailablePerTonne;
	nutrient.pigSlurryPTonnePerHa = PMean / nutrient.pigSlurryPAvailablePerTonne;
	nutrient.pigSlurryKTonnePerHa = KMean / nutrient.pigSlurryKAvailablePerTonne;
	nutrient.pigSlurryNLiterPerHa = 1000 * nutrient.pigSlurryNTonnePerHa / nutrient.slurryDensity;
	nutrient.pigSlurryPLiterPerHa = 1000 * nutrient.pigSlurryPTonnePerHa / nutrient.slurryDensity;
	nutrient.pigSlurryKLiterPerHa = 1000 * nutrient.pigSlurryKTonnePerHa / nutrient.slurryDensity;
	if isDebug then myPrintNutrientContent2("pigSlurry liters needed", textWidth2, nutrient.pigSlurryNLiterPerHa, nutrient.pigSlurryPLiterPerHa, nutrient.pigSlurryKLiterPerHa); end;

-- cows

	nutrient.cowSlurryNPerTonne = 2.600;	-- kg/Tonne
	nutrient.cowSlurryPPerTonne = 1.200;	-- kg/Tonne
	nutrient.cowSlurryKPerTonne = 2.500;	-- kg/Tonne
	if isDebug then myPrintNutrientContent("cowSlurry", textWidth2, nutrient.cowSlurryNPerTonne, nutrient.cowSlurryPPerTonne, nutrient.cowSlurryKPerTonne);  end;
	nutrient.cowSlurryNCropAvailability = 0.40;
	nutrient.cowSlurryPCropAvailability = 0.60;
	nutrient.cowSlurryKCropAvailability = 0.90;
	nutrient.cowSlurryNAvailablePerTonne = nutrient.cowSlurryNCropAvailability * nutrient.cowSlurryNPerTonne;
	nutrient.cowSlurryPAvailablePerTonne = nutrient.cowSlurryPCropAvailability * nutrient.cowSlurryPPerTonne;
	nutrient.cowSlurryKAvailablePerTonne = nutrient.cowSlurryKCropAvailability * nutrient.cowSlurryKPerTonne;
	if isDebug then myPrintNutrientContent("cowSlurry crop available", textWidth2, nutrient.cowSlurryNAvailablePerTonne, nutrient.cowSlurryPAvailablePerTonne, nutrient.cowSlurryKAvailablePerTonne); end;
	nutrient.cowSlurryNTonnePerHa = NMean / nutrient.cowSlurryNAvailablePerTonne;
	nutrient.cowSlurryPTonnePerHa = PMean / nutrient.cowSlurryPAvailablePerTonne;
	nutrient.cowSlurryKTonnePerHa = KMean / nutrient.cowSlurryKAvailablePerTonne;
	nutrient.cowSlurryNLiterPerHa = 1000 * nutrient.cowSlurryNTonnePerHa / nutrient.slurryDensity;
	nutrient.cowSlurryPLiterPerHa = 1000 * nutrient.cowSlurryPTonnePerHa / nutrient.slurryDensity;
	nutrient.cowSlurryKLiterPerHa = 1000 * nutrient.cowSlurryKTonnePerHa / nutrient.slurryDensity;
	if isDebug then myPrintNutrientContent2("cowSlurry liters needed", textWidth2, nutrient.cowSlurryNLiterPerHa, nutrient.cowSlurryPLiterPerHa, nutrient.cowSlurryKLiterPerHa); end;

-- Weighted values between pig and cow slurry

	nutrient.slurryNLiterPerHa = nutrient.pigSlurryFraction * nutrient.pigSlurryNLiterPerHa + nutrient.cowSlurryFraction * nutrient.cowSlurryNLiterPerHa;
	nutrient.slurryPLiterPerHa = nutrient.pigSlurryFraction * nutrient.pigSlurryPLiterPerHa + nutrient.cowSlurryFraction * nutrient.cowSlurryPLiterPerHa;
	nutrient.slurryKLiterPerHa = nutrient.pigSlurryFraction * nutrient.pigSlurryKLiterPerHa + nutrient.cowSlurryFraction * nutrient.cowSlurryKLiterPerHa;
	if isDebug then myPrintNutrientContent2("Mixed slurry liters needed", textWidth2, nutrient.slurryNLiterPerHa, nutrient.slurryPLiterPerHa, nutrient.slurryKLiterPerHa); end;

-- Fertilizer use split between organic and non-organic

	nutrient.reducedSlurryNLiterPerHa = nutrient.organicFertilizerFraction * nutrient.slurryNLiterPerHa;
	nutrient.reducedSlurryPLiterPerHa = nutrient.organicFertilizerFraction * nutrient.slurryPLiterPerHa;
	nutrient.reducedSlurryKLiterPerHa = nutrient.organicFertilizerFraction * nutrient.slurryKLiterPerHa;
	if isDebug then myPrintNutrientContent2("Reduced slurry liters needed", textWidth2, nutrient.reducedSlurryNLiterPerHa, nutrient.reducedSlurryPLiterPerHa, nutrient.reducedSlurryKLiterPerHa); end;
	if isDebug then print(" "); end;




-- Manure

--pigs

    nutrient.pigManureNPerTonne = 7.000;	-- kg/Tonne
	nutrient.pigManurePPerTonne = 6.000;	-- kg/Tonne
	nutrient.pigManureKPerTonne = 8.000;	-- kg/Tonne
	if isDebug then myPrintNutrientContent("pigManure", textWidth2, nutrient.pigManureNPerTonne, nutrient.pigManurePPerTonne, nutrient.pigManureKPerTonne); end;
	nutrient.pigManureNCropAvailability = 0.15;
	nutrient.pigManurePCropAvailability = 0.60;
	nutrient.pigManureKCropAvailability = 0.90;
	nutrient.pigManureNAvailablePerTonne = nutrient.pigManureNCropAvailability * nutrient.pigManureNPerTonne;
	nutrient.pigManurePAvailablePerTonne = nutrient.pigManurePCropAvailability * nutrient.pigManurePPerTonne;
	nutrient.pigManureKAvailablePerTonne = nutrient.pigManureKCropAvailability * nutrient.pigManureKPerTonne;
	if isDebug then myPrintNutrientContent("pigManure crop available", textWidth2, nutrient.pigManureNAvailablePerTonne, nutrient.pigManurePAvailablePerTonne, nutrient.pigManureKAvailablePerTonne); end;
	nutrient.pigManureNTonnePerHa = NMean / nutrient.pigManureNAvailablePerTonne;
	nutrient.pigManurePTonnePerHa = PMean / nutrient.pigManurePAvailablePerTonne;
	nutrient.pigManureKTonnePerHa = KMean / nutrient.pigManureKAvailablePerTonne;
	nutrient.pigManureNLiterPerHa = 1000 * nutrient.pigManureNTonnePerHa / nutrient.manureDensity;
	nutrient.pigManurePLiterPerHa = 1000 * nutrient.pigManurePTonnePerHa / nutrient.manureDensity;
	nutrient.pigManureKLiterPerHa = 1000 * nutrient.pigManureKTonnePerHa / nutrient.manureDensity;
	if isDebug then myPrintNutrientContent2("pigManure liters needed", textWidth2, nutrient.pigManureNLiterPerHa, nutrient.pigManurePLiterPerHa, nutrient.pigManureKLiterPerHa); end;


--cows

	nutrient.cowManureNPerTonne = 6.000;	-- kg/Tonne
	nutrient.cowManurePPerTonne = 3.200;	-- kg/Tonne
	nutrient.cowManureKPerTonne = 9.400;	-- kg/Tonne
	if isDebug then myPrintNutrientContent("cowManure", textWidth2, nutrient.cowManureNPerTonne, nutrient.cowManurePPerTonne, nutrient.cowManureKPerTonne); end;
	nutrient.cowManureNCropAvailability = 0.10;
	nutrient.cowManurePCropAvailability = 0.60;
	nutrient.cowManureKCropAvailability = 0.90;
	nutrient.cowManureNAvailablePerTonne = nutrient.cowManureNCropAvailability * nutrient.cowManureNPerTonne;
	nutrient.cowManurePAvailablePerTonne = nutrient.cowManurePCropAvailability * nutrient.cowManurePPerTonne;
	nutrient.cowManureKAvailablePerTonne = nutrient.cowManureKCropAvailability * nutrient.cowManureKPerTonne;
	if isDebug then myPrintNutrientContent("cowManure crop available", textWidth2, nutrient.cowManureNAvailablePerTonne, nutrient.cowManurePAvailablePerTonne, nutrient.cowManureKAvailablePerTonne); end;
	nutrient.cowManureNTonnePerHa = NMean / nutrient.cowManureNAvailablePerTonne;
	nutrient.cowManurePTonnePerHa = PMean / nutrient.cowManurePAvailablePerTonne;
	nutrient.cowManureKTonnePerHa = KMean / nutrient.cowManureKAvailablePerTonne;
	nutrient.cowManureNLiterPerHa = 1000 * nutrient.cowManureNTonnePerHa / nutrient.manureDensity;
	nutrient.cowManurePLiterPerHa = 1000 * nutrient.cowManurePTonnePerHa / nutrient.manureDensity;
	nutrient.cowManureKLiterPerHa = 1000 * nutrient.cowManureKTonnePerHa / nutrient.manureDensity;
	if isDebug then myPrintNutrientContent2("cowManure liters needed", textWidth2, nutrient.cowManureNLiterPerHa, nutrient.cowManurePLiterPerHa, nutrient.cowManureKLiterPerHa); end;

-- Weighted values between pig and cow manure

	nutrient.manureNLiterPerHa = nutrient.pigManureFraction * nutrient.pigManureNLiterPerHa + nutrient.cowManureFraction * nutrient.cowManureNLiterPerHa;
	nutrient.manurePLiterPerHa = nutrient.pigManureFraction * nutrient.pigManurePLiterPerHa + nutrient.cowManureFraction * nutrient.cowManurePLiterPerHa;
	nutrient.manureKLiterPerHa = nutrient.pigManureFraction * nutrient.pigManureKLiterPerHa + nutrient.cowManureFraction * nutrient.cowManureKLiterPerHa;
	if isDebug then myPrintNutrientContent2("Mixed manure liters needed", textWidth2, nutrient.manureNLiterPerHa, nutrient.manurePLiterPerHa, nutrient.manureKLiterPerHa); end;

-- Fertilizer use split between organic and non-organic

	nutrient.reducedManureNLiterPerHa = nutrient.organicFertilizerFraction * nutrient.manureNLiterPerHa;
	nutrient.reducedManurePLiterPerHa = nutrient.organicFertilizerFraction * nutrient.manurePLiterPerHa;
	nutrient.reducedManureKLiterPerHa = nutrient.organicFertilizerFraction * nutrient.manureKLiterPerHa;
	if isDebug then myPrintNutrientContent2("Reduced manure liters needed", textWidth2, nutrient.reducedManureNLiterPerHa, nutrient.reducedManurePLiterPerHa, nutrient.reducedManureKLiterPerHa); end;
	if isDebug then print(" "); end;

--Digestate

	nutrient.digestateNPerTonne = 4.867;	-- kg/Tonne
	nutrient.digestatePPerTonne = 1.050;	-- kg/Tonne
	nutrient.digestateKPerTonne = 3.450;	-- kg/Tonne
	if isDebug then myPrintNutrientContent("digestate", textWidth2, nutrient.digestateNPerTonne, nutrient.digestatePPerTonne, nutrient.digestateKPerTonne);  end;
	nutrient.digestateNCropAvailability = 0.55;
	nutrient.digestatePCropAvailability = 0.60;
	nutrient.digestateKCropAvailability = 0.90;
	nutrient.digestateNAvailablePerTonne = nutrient.digestateNCropAvailability * nutrient.digestateNPerTonne;
	nutrient.digestatePAvailablePerTonne = nutrient.digestatePCropAvailability * nutrient.digestatePPerTonne;
	nutrient.digestateKAvailablePerTonne = nutrient.digestateKCropAvailability * nutrient.digestateKPerTonne;
	if isDebug then myPrintNutrientContent("digestate crop available", textWidth2, nutrient.digestateNAvailablePerTonne, nutrient.digestatePAvailablePerTonne, nutrient.digestateKAvailablePerTonne); end;
	nutrient.digestateNTonnePerHa = NMean / nutrient.digestateNAvailablePerTonne;
	nutrient.digestatePTonnePerHa = PMean / nutrient.digestatePAvailablePerTonne;
	nutrient.digestateKTonnePerHa = KMean / nutrient.digestateKAvailablePerTonne;
	nutrient.digestateNLiterPerHa = 1000 * nutrient.digestateNTonnePerHa / nutrient.digestateDensity;
	nutrient.digestatePLiterPerHa = 1000 * nutrient.digestatePTonnePerHa / nutrient.digestateDensity;
	nutrient.digestateKLiterPerHa = 1000 * nutrient.digestateKTonnePerHa / nutrient.digestateDensity;
	if isDebug then myPrintNutrientContent2("digestate liters needed", textWidth2, nutrient.digestateNLiterPerHa, nutrient.digestatePLiterPerHa, nutrient.digestateKLiterPerHa); end;


-- Fertilizer use split between organic and non-organic

	nutrient.reducedDigestateNLiterPerHa = nutrient.organicFertilizerFraction * nutrient.digestateNLiterPerHa;
	nutrient.reducedDigestatePLiterPerHa = nutrient.organicFertilizerFraction * nutrient.digestatePLiterPerHa;
	nutrient.reducedDigestateKLiterPerHa = nutrient.organicFertilizerFraction * nutrient.digestateKLiterPerHa;

	if isDebug then myPrintNutrientContent2("Reduced digestate liters needed", textWidth2, nutrient.reducedDigestateNLiterPerHa, nutrient.reducedDigestatePLiterPerHa, nutrient.reducedDigestateKLiterPerHa); end;
	if isDebug then print("\nSpray rates set to meet phosphate (P2O5) requirements \n"); end;

	idx = 4;
	slurry.LitersPerHa = nutrient.reducedSlurryPLiterPerHa
	slurry.LitersPerSqm = slurry.LitersPerHa / 10000;
	slurry.sprayWidth = RN.sprayWidth[idx];															-- Spraying width in meters of slurry sprayer
	slurry.workingSpeed = RN.workingSpeed[idx];														-- Working speedlimit of slurry sprayer in kph
	slurry.sprayerCapacity = RN.sprayerCapacity[idx];													-- Sprayer capacity in liters
	slurry.AreaPerSec = slurry.sprayWidth * slurry.workingSpeed/3.6;				-- Area covered per second
	slurry.workCapacity = 3600 * slurry.AreaPerSec/10000;							-- ha per hour
	slurry.litersPerSecond = slurry.AreaPerSec * slurry.LitersPerSqm;
	slurry.calibrationDose = calibrationTime * slurry.litersPerSecond;				-- Target dose spread in calibration time
	slurry.calibrationArea = calibrationTime * slurry.AreaPerSec;					-- Area covered n calibration time
	slurry.calibrationFactor = RN.sprayCalibrationFactor[idx];
	sprayRates[idx] = slurry.calibrationFactor * slurry.LitersPerSqm/3.6;
	slurry.haPerLoad = slurry.sprayerCapacity / slurry.LitersPerHa;
	RN.sprayVolume[idx] = slurry.LitersPerHa;

	idx = 5;
	digestate.LitersPerHa = nutrient.reducedDigestatePLiterPerHa
	digestate.LitersPerSqm = digestate.LitersPerHa / 10000;
	digestate.sprayWidth = RN.sprayWidth[idx];														-- Spraying width in meters of digestate sprayer
	digestate.workingSpeed = RN.workingSpeed[idx];													-- Working speedlimit of digestate sprayer in kph
	digestate.sprayerCapacity = RN.sprayerCapacity[idx];												-- Sprayer capacity in liters
	digestate.AreaPerSec = digestate.sprayWidth * digestate.workingSpeed/3.6;
	digestate.workCapacity = 3600*digestate.AreaPerSec/10000;						-- ha per hour
	digestate.litersPerSecond = digestate.AreaPerSec * digestate.LitersPerSqm;
	digestate.calibrationDose = calibrationTime * digestate.litersPerSecond;		-- Target dose spread in calibration time
	digestate.calibrationArea = calibrationTime * digestate.AreaPerSec;				-- Area covered n calibration time
	digestate.calibrationFactor = RN.sprayCalibrationFactor[idx];
	sprayRates[idx] = digestate.calibrationFactor * digestate.LitersPerSqm/3.6;
	digestate.haPerLoad = digestate.sprayerCapacity / digestate.LitersPerHa;
	RN.sprayVolume[idx] = digestate.LitersPerHa;

	idx = 3;
	manure.LitersPerHa = nutrient.reducedManurePLiterPerHa;
	manure.LitersPerSqm = manure.LitersPerHa / 10000;
	manure.sprayWidth = RN.sprayWidth[idx];															-- Spraying width in meters of manure sprayer
	manure.workingSpeed = RN.workingSpeed[idx];														-- Working speedlimit of manure sprayer in kph
	manure.sprayerCapacity = RN.sprayerCapacity[idx];													-- Sprayer capacity in liters
	manure.AreaPerSec = manure.sprayWidth * manure.workingSpeed/3.6;
	manure.workCapacity = 3600*manure.AreaPerSec/10000;								-- ha per hour
	manure.litersPerSecond = manure.AreaPerSec * manure.LitersPerSqm;
	manure.calibrationDose = calibrationTime * manure.litersPerSecond;				-- Target dose spread in calibration time
	manure.calibrationArea = calibrationTime * manure.AreaPerSec;					-- Area covered n calibration time
	manure.calibrationFactor = RN.sprayCalibrationFactor[idx];
	sprayRates[idx] = manure.calibrationFactor * manure.LitersPerSqm/3.6;
	manure.haPerLoad = manure.sprayerCapacity / manure.LitersPerHa;
	RN.sprayVolume[idx] = manure.LitersPerHa;


-- actually delivered organinc N, P, K in kg/ha

-- slurry

	nutrient.slurrySprayLiterPerHa = nutrient.reducedSlurryPLiterPerHa;										-- Actual slurry dose/ha
	nutrient.pigSlurrySprayLiterPerHa = nutrient.pigSlurryFraction * nutrient.slurrySprayLiterPerHa; 		-- pig slurry fraction of dose in Liter/ha
	nutrient.cowSlurrySprayLiterPerHa = nutrient.cowSlurryFraction * nutrient.slurrySprayLiterPerHa; 		-- cow fraction of dose in Liter/ha
	nutrient.pigSlurrySprayTonnePerHa = nutrient.slurryDensity * nutrient.pigSlurrySprayLiterPerHa/1000; 	-- pig slurry fraction of dose in Tonne/ha
	nutrient.cowSlurrySprayTonnePerHa = nutrient.slurryDensity * nutrient.cowSlurrySprayLiterPerHa/1000; 	-- cow slurry fraction of dose in Tonne/ha

	nutrient.NSlurrySpraykgPerHa = nutrient.pigSlurryNAvailablePerTonne * nutrient.pigSlurrySprayTonnePerHa + nutrient.cowSlurryNAvailablePerTonne * nutrient.cowSlurrySprayTonnePerHa;
	nutrient.PSlurrySpraykgPerHa = nutrient.pigSlurryPAvailablePerTonne * nutrient.pigSlurrySprayTonnePerHa + nutrient.cowSlurryPAvailablePerTonne * nutrient.cowSlurrySprayTonnePerHa;
	nutrient.KSlurrySpraykgPerHa = nutrient.pigSlurryKAvailablePerTonne * nutrient.pigSlurrySprayTonnePerHa + nutrient.cowSlurryKAvailablePerTonne * nutrient.cowSlurrySprayTonnePerHa;

--manure

	nutrient.manureSpreadLiterPerHa = nutrient.reducedManurePLiterPerHa;									-- Actual manure dose/ha
	nutrient.pigManureSpreadLiterPerHa = nutrient.pigManureFraction * nutrient.manureSpreadLiterPerHa; 		-- pig manure fraction of dose in Liter/ha
	nutrient.cowManureSpreadLiterPerHa = nutrient.cowManureFraction * nutrient.manureSpreadLiterPerHa; 		-- cow fraction of dose in Liter/ha
	nutrient.pigManureSpreadTonnePerHa = nutrient.manureDensity * nutrient.pigManureSpreadLiterPerHa/1000; 	-- pig manure fraction of dose in Tonne/ha
	nutrient.cowManureSpreadTonnePerHa = nutrient.manureDensity * nutrient.cowManureSpreadLiterPerHa/1000; 	-- cow manure fraction of dose in Tonne/ha

	nutrient.NManureSpreadkgPerHa = nutrient.pigManureNAvailablePerTonne * nutrient.pigManureSpreadTonnePerHa + nutrient.cowManureNAvailablePerTonne * nutrient.cowManureSpreadTonnePerHa;
	nutrient.PManureSpreadkgPerHa = nutrient.pigManurePAvailablePerTonne * nutrient.pigManureSpreadTonnePerHa + nutrient.cowManurePAvailablePerTonne * nutrient.cowManureSpreadTonnePerHa;
	nutrient.KManureSpreadkgPerHa = nutrient.pigManureKAvailablePerTonne * nutrient.pigManureSpreadTonnePerHa + nutrient.cowManureKAvailablePerTonne * nutrient.cowManureSpreadTonnePerHa;

-- digestate

	nutrient.digestateSprayLiterPerHa = nutrient.reducedDigestatePLiterPerHa;
	nutrient.digestateSprayTonnePerHa = nutrient.digestateDensity * nutrient.digestateSprayLiterPerHa/1000;

	nutrient.NDigestateSpraykgPerHa = nutrient.digestateNAvailablePerTonne * nutrient.digestateSprayTonnePerHa;
	nutrient.PDigestateSpraykgPerHa = nutrient.digestatePAvailablePerTonne * nutrient.digestateSprayTonnePerHa;
	nutrient.KDigestateSpraykgPerHa = nutrient.digestateKAvailablePerTonne * nutrient.digestateSprayTonnePerHa;

--artificial fertilizer

	nutrient.artificialNkgPerHa = NMean - (nutrient.NSlurrySpraykgPerHa + nutrient.NManureSpreadkgPerHa + nutrient.NDigestateSpraykgPerHa)/3;
	nutrient.artificialPkgPerHa = PMean - (nutrient.PSlurrySpraykgPerHa + nutrient.PManureSpreadkgPerHa + nutrient.PDigestateSpraykgPerHa)/3;
	nutrient.artificialKkgPerHa = KMean - (nutrient.KSlurrySpraykgPerHa + nutrient.KManureSpreadkgPerHa + nutrient.KDigestateSpraykgPerHa)/3;

--- NPK 14:7:14
	nutrient.artificialkgPerHa = nutrient.artificialKkgPerHa/nutrient.artificialKFraction;
	nutrient.artificialNSpreadkgPerHa = nutrient.artificialNFraction * nutrient.artificialkgPerHa;
	nutrient.artificialPSpreadkgPerHa = nutrient.artificialPFraction * nutrient.artificialkgPerHa;
	nutrient.artificialKSpreadkgPerHa = nutrient.artificialKFraction * nutrient.artificialkgPerHa;

	if isDebug then
		print("\n---Delivered nutrients---\n");
	end;
	if isDebug then
		myPrintSprayInfo2("Mean requirement", textWidth2, NMean, PMean, KMean);
		myPrintSprayInfo2("Slurry source", textWidth2, nutrient.NSlurrySpraykgPerHa, nutrient.PSlurrySpraykgPerHa, nutrient.KSlurrySpraykgPerHa);
		myPrintSprayInfo2("Manure source", textWidth2, nutrient.NManureSpreadkgPerHa, nutrient.PManureSpreadkgPerHa, nutrient.KManureSpreadkgPerHa);
		myPrintSprayInfo2("Digestate source", textWidth2, nutrient.NDigestateSpraykgPerHa, nutrient.PDigestateSpraykgPerHa, nutrient.KDigestateSpraykgPerHa);
		myPrintSprayInfo2("Artificial minimum", textWidth2, nutrient.artificialNkgPerHa, nutrient.artificialPkgPerHa, nutrient.artificialKkgPerHa);
		myPrintSprayInfo2("Artificial given", textWidth2, nutrient.artificialNSpreadkgPerHa, nutrient.artificialPSpreadkgPerHa, nutrient.artificialKSpreadkgPerHa);
	end;

-- Solid fertilizer

	idx = 1;

	fertilizer.kgPerHa = nutrient.artificialkgPerHa;
	fertilizer.LitersPerHa = fertilizer.kgPerHa/sprayTypeDensity[idx];
	fertilizer.LitersPerSqm = fertilizer.LitersPerHa / 10000;
	fertilizer.sprayWidth = RN.sprayWidth[idx];														-- Spraying width in meters of fertilizer sprayer
	fertilizer.workingSpeed = RN.workingSpeed[idx];													-- Working speedlimit of fertilizer sprayer in kph
	fertilizer.sprayerCapacity = RN.sprayerCapacity[idx];											-- Sprayer capacity in liters
	fertilizer.AreaPerSec = fertilizer.sprayWidth * fertilizer.workingSpeed/3.6;
	fertilizer.workCapacity = 3600*fertilizer.AreaPerSec/10000;						-- ha per hour
	fertilizer.litersPerSecond = fertilizer.AreaPerSec * fertilizer.LitersPerSqm;
	fertilizer.calibrationDose = calibrationTime * fertilizer.litersPerSecond;		-- Target dose spread in calibration time
	fertilizer.calibrationArea = calibrationTime * fertilizer.AreaPerSec;			-- Area covered n calibration time
	fertilizer.calibrationFactor = RN.sprayCalibrationFactor[idx];
	sprayRates[idx] = fertilizer.calibrationFactor * fertilizer.LitersPerSqm/3.6;
	fertilizer.haPerLoad = fertilizer.sprayerCapacity / fertilizer.LitersPerHa;
	RN.sprayVolume[idx] = fertilizer.LitersPerHa

--Liquid fertilizer

	idx = 2;
	liqfert.kgPerHa = liqfert.liquidReductionFactor * nutrient.artificialkgPerHa;
	liqfert.concentrateLitersPerHa = liqfert.kgPerHa/sprayTypeDensity[idx];
	liqfert.LitersPerHa = liqfert.dilutionFactor * liqfert.concentrateLitersPerHa;
	liqfert.LitersPerSqm = liqfert.LitersPerHa/10000;
	liqfert.sprayWidth = RN.sprayWidth[idx];														-- Spraying width in meters of liqfert sprayer
	liqfert.workingSpeed = RN.workingSpeed[idx];														-- Working speedlimit of liqfert sprayer in kph
	liqfert.sprayerCapacity = RN.sprayerCapacity[idx];													-- Sprayer capacity in liters
	liqfert.AreaPerSec = liqfert.sprayWidth * liqfert.workingSpeed/3.6;
	liqfert.workCapacity = 3600*liqfert.AreaPerSec/10000;							-- ha per hour
	liqfert.litersPerSecond = liqfert.AreaPerSec * liqfert.LitersPerSqm;
	liqfert.calibrationDose = calibrationTime * liqfert.litersPerSecond;			-- Target dose spread in calibration time
	liqfert.calibrationArea = calibrationTime * liqfert.AreaPerSec;					-- Area covered n calibration time
	liqfert.calibrationFactor = RN.sprayCalibrationFactor[idx];		 										-- 0.697; (36 m)
	sprayRates[idx] = liqfert.calibrationFactor * liqfert.LitersPerSqm/3.6;
	liqfert.haPerLoad = liqfert.sprayerCapacity / liqfert.LitersPerHa;
	RN.sprayVolume[idx] = liqfert.LitersPerHa;

-- LIME

	soil.Name = {"Sands and loamy sands", "Sandy loams and silt loam", "Clay loams and clay"};
	soil.Type = RN.soilType;				-- 1: Sands and loamy sands: 2: Sandy loams and silt loam; 3: Clay loams and clay
	soil.currentpH = RN.soilCurrentpH;
	soil.targetpH = RN.soilTargetpH;

	idx = 6;
	lime.factor = {6, 7, 8};
	lime.NV = RN.limeNV;  																	-- concentration of calcium in lime material, 0 - 100
	lime.kgPerHa = 1000 * 50 *(soil.targetpH - soil.currentpH)*lime.factor[soil.Type]/lime.NV;
	lime.LitersPerHa = lime.kgPerHa / sprayTypeDensity[idx];
	lime.LitersPerSqm = lime.LitersPerHa / 10000;
	lime.sprayWidth = RN.sprayWidth[idx];															-- Spraying width in meters of lime sprayer
	lime.workingSpeed = RN.workingSpeed[idx];															-- Working speedlimit of lime sprayer in kph
	lime.sprayerCapacity = RN.sprayerCapacity[idx];													-- Sprayer capacity in liters
	lime.AreaPerSec = lime.sprayWidth * lime.workingSpeed/3.6;
	lime.workCapacity = 3600*lime.AreaPerSec/10000;									-- ha per hour
	lime.litersPerSecond = lime.AreaPerSec * lime.LitersPerSqm;
	lime.calibrationDose = calibrationTime * lime.litersPerSecond;					-- Target dose spread in calibration time
	lime.calibrationArea = calibrationTime * lime.AreaPerSec;						-- Area covered n calibration time
	lime.calibrationFactor = RN.sprayCalibrationFactor[idx];
	sprayRates[idx] = lime.calibrationFactor * lime.LitersPerSqm/3.6;
	lime.haPerLoad = lime.sprayerCapacity / lime.LitersPerHa;
	RN.sprayVolume[idx] = lime.LitersPerHa;

-- Herbicide

	idx = 7;
	herbicide.kgPerHa = RN.HerbicideKgPerHa;
	herbicide.LitersPerHa = herbicide.dilutionFactor * herbicide.kgPerHa / sprayTypeDensity[idx];
	herbicide.LitersPerSqm = herbicide.LitersPerHa / 10000;
	herbicide.sprayWidth = RN.sprayWidth[idx];														-- Spraying width in meters of herbicide sprayer
	herbicide.workingSpeed = RN.workingSpeed[idx];													-- Working speedlimit of herbicide sprayer in kph
	herbicide.sprayerCapacity = RN.sprayerCapacity[idx];												-- Sprayer capacity in liters
	herbicide.AreaPerSec = herbicide.sprayWidth * herbicide.workingSpeed/3.6;
	herbicide.workCapacity = 3600*herbicide.AreaPerSec/10000;						-- ha per hour
	herbicide.litersPerSecond = herbicide.AreaPerSec * herbicide.LitersPerSqm;
	herbicide.calibrationDose = calibrationTime * herbicide.litersPerSecond;		-- Target dose spread in calibration time
	herbicide.calibrationArea = calibrationTime * herbicide.AreaPerSec;				-- Area covered n calibration time
	herbicide.calibrationFactor = RN.sprayCalibrationFactor[idx];
	sprayRates[idx] = herbicide.calibrationFactor * herbicide.LitersPerSqm/3.6;
	herbicide.haPerLoad = herbicide.sprayerCapacity / herbicide.LitersPerHa;
	RN.sprayVolume[idx] = herbicide.LitersPerHa;

	if isDebug then
		print("\n---Lime---\n");
		myPrint("Spray width",lime.sprayWidth,textWidth3, " m");
		myPrint("Working speed",lime.workingSpeed,textWidth3, " kph");
		myPrint("Sprayer capacity",lime.sprayerCapacity,textWidth3, " L");
		myPrintFloat("Work capacity",lime.workCapacity,textWidth3, " Ha/hour");
		myPrint("Lime liter per ha:", lime.LitersPerHa,textWidth3, " L/Ha");
		myPrintFloat("Liters per square meter",lime.LitersPerSqm,textWidth3, " L/sqm");
		myPrintFloat("Square meters per sec",lime.AreaPerSec,textWidth3, " sqm/s");
		myPrintFloat("Liters per sec",lime.litersPerSecond,textWidth3, " L/s");
		myPrintFloat("Ha per load",lime.haPerLoad,textWidth3, " Ha");
		myPrint("Calibration time",calibrationTime,textWidth3, " sec");
		myPrint("Calibration dose",lime.calibrationDose,textWidth3, " L");
		myPrint("Calibration area",lime.calibrationArea,textWidth3, " sqm");
		myPrintFloat("Calibration factor",lime.calibrationFactor,textWidth3, " ");
	end;


	if isDebug then
		print("\n---Slurry---\n");
		myPrint("Spray width",slurry.sprayWidth,textWidth3, " m");
		myPrint("Working speed",slurry.workingSpeed,textWidth3, " kph");
		myPrint("Sprayer capacity",slurry.sprayerCapacity,textWidth3, " L");
		myPrintFloat("Work capacity",slurry.workCapacity,textWidth3, " Ha/hour");
		myPrint("Slurry liter per ha:", slurry.LitersPerHa,textWidth3, " L/Ha");
		myPrintFloat("Liters per square meter",slurry.LitersPerSqm,textWidth3, " L/sqm");
		myPrintFloat("Square meters per sec",slurry.AreaPerSec,textWidth3, " sqm/s");
		myPrintFloat("Liters per sec",slurry.litersPerSecond,textWidth3, " L/s");
		myPrintFloat("Ha per load",slurry.haPerLoad,textWidth3, " Ha");
		myPrint("Calibration time",calibrationTime,textWidth3, " sec");
		myPrint("Calibration dose",slurry.calibrationDose,textWidth3, " L");
		myPrint("Calibration area",slurry.calibrationArea,textWidth3, " sqm");
		myPrintFloat("Calibration factor",slurry.calibrationFactor,textWidth3, " ");
	end;

	if isDebug then
		print("\n---Manure---\n");
		myPrint("Spray width",manure.sprayWidth,textWidth3, " m");
		myPrint("Working speed",manure.workingSpeed,textWidth3, " kph");
		myPrint("Sprayer capacity",manure.sprayerCapacity,textWidth3, " L");
		myPrintFloat("Work capacity",manure.workCapacity,textWidth3, " Ha/hour");
		myPrint("Manure liter per ha:", manure.LitersPerHa,textWidth3, " L/Ha");
		myPrintFloat("Liters per square meter",manure.LitersPerSqm,textWidth3, " L/sqm");
		myPrintFloat("Square meters per sec",manure.AreaPerSec,textWidth3, " sqm/s");
		myPrintFloat("Liters per sec",manure.litersPerSecond,textWidth3, " L/s");
		myPrintFloat("Ha per load",manure.haPerLoad,textWidth3, " Ha");
		myPrint("Calibration time",calibrationTime,textWidth3, " sec");
		myPrint("Calibration dose",manure.calibrationDose,textWidth3, " L");
		myPrint("Calibration area",manure.calibrationArea,textWidth3, " sqm");
		myPrintFloat("Calibration factor",manure.calibrationFactor,textWidth3, " ");
	end;

	if isDebug then
		print("\n---Digestate---\n");
		myPrint("Spray width",digestate.sprayWidth,textWidth3, " m");
		myPrint("Working speed",digestate.workingSpeed,textWidth3, " kph");
		myPrint("Sprayer capacity",digestate.sprayerCapacity,textWidth3, " L");
		myPrintFloat("Work capacity",digestate.workCapacity,textWidth3, " Ha/hour");
		myPrint("Digestate liter per ha:", digestate.LitersPerHa,textWidth3, " L/Ha");
		myPrintFloat("Liters per square meter",digestate.LitersPerSqm,textWidth3, " L/sqm");
		myPrintFloat("Square meters per sec",digestate.AreaPerSec,textWidth3, " sqm/s");
		myPrintFloat("Liters per sec",digestate.litersPerSecond,textWidth3, " L/s");
		myPrintFloat("Ha per load",digestate.haPerLoad,textWidth3, " Ha");
		myPrint("Calibration time",calibrationTime,textWidth3, " sec");
		myPrint("Calibration dose",digestate.calibrationDose,textWidth3, " L");
		myPrint("Calibration area",digestate.calibrationArea,textWidth3, " sqm");
		myPrintFloat("Calibration factor",digestate.calibrationFactor,textWidth3, " ");
	end;

	if isDebug then
		print("\n---Fertilizer---\n");
		myPrint("Spray width",fertilizer.sprayWidth,textWidth3, " m");
		myPrint("Working speed",fertilizer.workingSpeed,textWidth3, " kph");
		myPrint("Sprayer capacity",fertilizer.sprayerCapacity,textWidth3, " L");
		myPrintFloat("Work capacity",fertilizer.workCapacity,textWidth3, " Ha/hour");
		myPrint("Fertilizer liter per ha:", fertilizer.LitersPerHa,textWidth3, " L/Ha");
		myPrintFloat("Liters per square meter",fertilizer.LitersPerSqm,textWidth3, " L/sqm");
		myPrintFloat("Square meters per sec",fertilizer.AreaPerSec,textWidth3, " sqm/s");
		myPrintFloat("Liters per sec",fertilizer.litersPerSecond,textWidth3, " L/s");
		myPrintFloat("Ha per load",fertilizer.haPerLoad,textWidth3, " Ha");
		myPrint("Calibration time",calibrationTime,textWidth3, " sec");
		myPrint("Calibration dose",fertilizer.calibrationDose,textWidth3, " L");
		myPrint("Calibration area",fertilizer.calibrationArea,textWidth3, " sqm");
		myPrintFloat("Calibration factor",fertilizer.calibrationFactor,textWidth3, " ");
	end;

	if isDebug then
		print("\n---Liquid fertilizer---\n");
		myPrint("Spray width",liqfert.sprayWidth,textWidth3, " m");
		myPrint("Working speed",liqfert.workingSpeed,textWidth3, " kph");
		myPrint("Sprayer capacity",liqfert.sprayerCapacity,textWidth3, " L");
		myPrintFloat("Work capacity",liqfert.workCapacity,textWidth3, " Ha/hour");
		myPrint("Diluted fertilizer liter per ha:", liqfert.LitersPerHa,textWidth3, " L/Ha");
		myPrintFloat("Liters per square meter",liqfert.LitersPerSqm,textWidth3, " L/sqm");
		myPrintFloat("Square meters per sec",liqfert.AreaPerSec,textWidth3, " sqm/s");
		myPrintFloat("Liters per sec",liqfert.litersPerSecond,textWidth3, " L/s");
		myPrintFloat("Ha per load",liqfert.haPerLoad,textWidth3, " Ha");
		myPrint("Calibration time",calibrationTime,textWidth3, " sec");
		myPrint("Calibration dose",liqfert.calibrationDose,textWidth3, " L");
		myPrint("Calibration area",liqfert.calibrationArea,textWidth3, " sqm");
		myPrintFloat("Calibration factor",liqfert.calibrationFactor,textWidth3, " ");
	end;

	if isDebug then
		print("\n---Herbicide---\n");
		myPrint("Spray width",herbicide.sprayWidth,textWidth3, " m");
		myPrint("Working speed",herbicide.workingSpeed,textWidth3, " kph");
		myPrint("Sprayer capacity",herbicide.sprayerCapacity,textWidth3, " L");
		myPrintFloat("Work capacity",herbicide.workCapacity,textWidth3, " Ha/hour");
		myPrint("Diluted herbicide liter per ha:", herbicide.LitersPerHa,textWidth3, " L/Ha");
		myPrintFloat("Liters per square meter",herbicide.LitersPerSqm,textWidth3, " L/sqm");
		myPrintFloat("Square meters per sec",herbicide.AreaPerSec,textWidth3, " sqm/s");
		myPrintFloat("Liters per sec",herbicide.litersPerSecond,textWidth3, " L/s");
		myPrintFloat("Ha per load",herbicide.haPerLoad,textWidth3, " Ha");
		myPrint("Calibration time",calibrationTime,textWidth3, " sec");
		myPrint("Calibration dose",herbicide.calibrationDose,textWidth3, " L");
		myPrint("Calibration area",herbicide.calibrationArea,textWidth3, " sqm");
		myPrintFloat("Calibration factor",herbicide.calibrationFactor,textWidth3, " ");
	end;

	if isDebug then print("\n---Game spray parameters---\n"); end;
	if g_sprayTypeManager.sprayTypes  ~= nil then
		for k,sprayType in pairs(g_sprayTypeManager.sprayTypes) do
			local fillType = sprayType.fillType;
			fillType.startPricePerLiter  = fillType.pricePerLiter;
			fillType.previousHourPrice   = fillType.pricePerLiter;
			sprayType.litersPerSecond  = sprayRates[k];
			local sprayTypeName  = sprayType.name;
			if isDebug then myPrintFloat(sprayTypeName, sprayType.litersPerSecond,textWidth3, "Liter/sec"); end;
		end;
	end;

	if isDebug then print("\nRealNumbersSpraying Debug Report End \n"); end;
end;

function RealNumbersSpraying:keyEvent(unicode, sym, modifier, isDown)
	local isDebug = RN.isDebug;
	local textWidth3 = 40;
	if Input.isKeyPressed(Input.KEY_lalt) and Input.isKeyPressed(Input.KEY_r) and Input.isKeyPressed(Input.KEY_s) then
		if isDebug then
			print("\nRealNumbersSpraying Quick Report\n")
			print("Left Alt-rs pressed\n");			
			myPrint("Fertilizer liter per ha:", RN.sprayVolume[1],textWidth3, " L/Ha");
			myPrint("Diluted fertilizer liter per ha:", RN.sprayVolume[2],textWidth3, " L/Ha");
			myPrint("Manure liter per ha:", RN.sprayVolume[3],textWidth3, " L/Ha");
			myPrint("Slurry liter per ha:", RN.sprayVolume[4],textWidth3, " L/Ha");
			myPrint("Digestate liter per ha:", RN.sprayVolume[5],textWidth3, " L/Ha");
			myPrint("Lime liter per ha:", RN.sprayVolume[6],textWidth3, " L/Ha");
			myPrint("Diluted herbicide liter per ha:", RN.sprayVolume[7],textWidth3, " L/Ha\n");			
		end;
	end;
end;
function RealNumbersSpraying:update(dt)
end;
function RealNumbersSpraying:draw()
end;
function RealNumbersSpraying:deleteMap()
end;
function RealNumbersSpraying:mouseEvent(posX, posY, isDown, isUp, button)
end;

addModEventListener(RealNumbersSpraying);
